c*******************************************************************
c* Short description                                               *
c*   Library to interpolate material-related transport data        *
c*                                                                 *
c* Dependencies:                                                   *
c*   -> uses libelastic.f                                          *
c*   -> uses libeloss.f                                            *
c*   -> uses libmath.f                                             *
c*                                                                 *
c* Last update                                                     *
c*   2000-04-23                                                    *
c*******************************************************************

c*******************************************************************
c*   Interpolation of raw input data                               *
c*******************************************************************
c*******************************************************************


      subroutine inimat
     &(Uatno,Uatno2,Uatn21,Umass,Udens,Uexcite,einf,esup,epinf,epsup)
c*******************************************************************
c*    Creates log cubic spline coefs. from data in PENELOPE's      *
c*    format *.mat. These coefs can be further used to calculate   *
c*    interpolations of several relevant transport quantities      *
c*                                                                 *
c*    Input:                                                       *
c*      reads data from unit 10                                    *
c*    Output:                                                      *
c*      outputs to unit 11, and, additionally,                     *
c*      atno -> total atomic number of the molecule                *
c*      atno2 -> sum of squares of atomic numbers (Z) in molecule  *
c*      atno21 -> sum of Z(Z+1) in the molecule                    *
c*      mass -> molecular weight of the given material in amu      *
c*      dens -> excitation density effect (g/cm^3)                 *
c*      excite -> mean excitation energy (eV)                      *
c*      einf -> minimum e- energy in input data tables (eV)        *
c*      esup -> maximum e- energy in input data tables (eV)        *
c*      epinf -> minimum photon energy in input data tables (eV)   *
c*      epsup -> maximum photon energy in input data tables (eV)   *
c*    Comments:                                                    *
c*      -> for compounds, atno2 is the stoichiometric sum of the   *
c*         squares of the components                               *
c*      -> both units 10 & 11 must be already open before the call;*
c*         units are not closed when routine is done               *
c*******************************************************************
      implicit none
      real*8 Uatno,Uatno2,Uatn21,Umass,Udens,Uexcite
      real*8 einf,esup,epinf,epsup

      real*8 atno,atno2,atno21,mass,dens,excite
      common /mater/ atno,atno2,atno21,mass,dens,excite
      real*8 zasum
      common /mater2/ zasum
      integer mad,ndata
      parameter (mad=200)
      real*8 elmfp,elmfpa,elmfpb,elmfpc,elmfpd
      real*8 tmfp1,tmfp1a,tmfp1b,tmfp1c,tmfp1d
      real*8 tmfp2,tmfp2a,tmfp2b,tmfp2c,tmfp2d
      real*8 cstop,cstopa,cstopb,cstopc,cstopd
      real*8 rstop,rstopa,rstopb,rstopc,rstopd
      real*8 range,rangea,rangeb,rangec,ranged,
     &             eranga,erangb,erangc,erangd
      real*8 awpar,awpara,awparb,awparc,awpard
      real*8 energy
      common /rawipc/
     &  elmfp(mad),elmfpa(mad),elmfpb(mad),elmfpc(mad),elmfpd(mad),
     &  tmfp1(mad),tmfp1a(mad),tmfp1b(mad),tmfp1c(mad),tmfp1d(mad),
     &  tmfp2(mad),tmfp2a(mad),tmfp2b(mad),tmfp2c(mad),tmfp2d(mad),
     &  cstop(mad),cstopa(mad),cstopb(mad),cstopc(mad),cstopd(mad),
     &  rstop(mad),rstopa(mad),rstopb(mad),rstopc(mad),rstopd(mad),
     &  range(mad),rangea(mad),rangeb(mad),rangec(mad),ranged(mad),
     &             eranga(mad),erangb(mad),erangc(mad),erangd(mad),
     &  awpar(mad),awpara(mad),awparb(mad),awparc(mad),awpard(mad),
     &  energy(mad),ndata
      integer ndatph,ndatpp
      real*8 enepho,rayl,cmpt,phot,epairp,pair
      common /tabpho/
     &  enepho(mad),rayl(mad),cmpt(mad),phot(mad),
     &  epairp(mad),pair(mad),ndatph,ndatpp
      real*8
     &  rayla,raylb,raylc,rayld,cmpta,cmptb,cmptc,cmptd,
     &  phota,photb,photc,photd,paira,pairb,pairc,paird
      common /ippho/
     &  rayla(mad),raylb(mad),raylc(mad),rayld(mad),
     &  cmpta(mad),cmptb(mad),cmptc(mad),cmptd(mad),
     &  phota(mad),photb(mad),photc(mad),photd(mad),
     &  paira(mad),pairb(mad),pairc(mad),paird(mad)
      real*8 atm
      common /catmas/ atm(99)
      real*8 mee
      common /cmexce/ mee(99)

      character*100 buffer
      integer i,noelem,eatno,ipairs,nos,newi,i1
      real*8 e0,e1,drang,awsolv,accu,stoic,eps,e,mc2,istop(mad)
      parameter (accu=1.d-10,eps=1.d-10,mc2=510.9991d3)
      real*8 istopa(mad),istopb(mad),istopc(mad),istopd(mad)

      read(10,'(a100)') buffer
      read(10,1030) dens
 1030 format(19X,E11.4)
      write(11,*) 'Reference density (g/cm^3):'
      write(11,'(1x,1pe14.7)') dens
      read(10,1040) noelem
 1040 format(41X,I2)
      write(11,*) 'No elements in molecule:'
      write(11,'(1x,i3)') noelem
      atno = 0.d0
      atno2 = 0.d0
      atno21 = 0.d0
      mass = 0.d0
      excite = 0.d0
      zasum = 0.d0
      do 10 i=1,noelem
        read(10,1050) eatno,stoic
 1050   format(20x,i2,18x,e13.6)
        write(11,'(1x,i5,1x,1pe14.7))') eatno,stoic
        atno = atno+stoic*eatno
        atno2 = atno2+stoic*eatno**2
        atno21 = atno21+stoic*eatno*(eatno+1)
        mass = mass+stoic*atm(eatno)
        zasum = zasum+stoic*eatno*atm(eatno)
        excite = excite+stoic*eatno*dlog(mee(eatno))
 10   continue
      write(11,*) 'Total atno(Z), Z^2 & Z(Z+1):'
      write(11,'(1x,3(1pe14.7,1x))') atno,atno2,atno21
      write(11,*) 'Atomic mass (amu):'
      write(11,'(1x,1pe14.7)') mass
      excite = dexp(excite/atno)
      Uatno  = atno
      Uatno2 = atno2
      Uatn21 = atno21
      Umass  = mass
      Udens  = dens
      Uexcite= excite

      read(10,'(a100)') buffer
      read(10,1070) nos
 1070 format(28x,i2)
      do 4 i=1,nos
        read(10,'(a100)') buffer
    4 continue

      read(10,1001) ndata
 1001 format(31x,i3)
      if (ndata.gt.mad.or.ndata.lt.10) then
        write(*,*) 'inimat Error: invalid ndata: '
        write(*,'(1x,i5)') ndata
        stop
      endif

      do 20 i=1,ndata
        read(10,*) energy(i),elmfp(i),tmfp1(i),tmfp2(i),cstop(i),
     &            rstop(i)
        if (i.eq.1) einf=energy(i)
        if (i.eq.ndata) esup=energy(i)
        elmfp(i) = 1.d-6*elmfp(i)
        tmfp1(i) = 1.d-6*tmfp1(i)
        tmfp2(i) = 1.d-6*tmfp2(i)
        cstop(i) = 1.d6*cstop(i)
        rstop(i) = 1.d6*rstop(i)
        istop(i) = 1.d0/(rstop(i)+cstop(i))
        awpar(i) = awsolv(elmfp(i)/tmfp1(i),accu)
 20   continue
      write(*,1010) ndata
 1010 format (1x,'inimat: ',i4,' data successfully read')
      call spline(energy,istop,istopa,istopb,istopc,istopd,
     &            0.d0,0.d0,ndata)
com: Get the range from the integral of 1/TotalStoppingPower:
      range(1) = 0.d0
      do 25 i=1,ndata-1
        e0 = energy(i)
        e1 = energy(i+1)
        call integ(energy,istopa,istopb,istopc,istopd,e0,e1,drang,
     &             ndata)
        range(i+1) = range(i)+drang
 25   continue
com: Preparing log-log interpolation:
      do 30 i=1,ndata
        e = energy(i)
        energy(i)= dlog(energy(i))
        elmfp(i) = dlog(elmfp(i))
        tmfp1(i) = dlog(tmfp1(i))
        tmfp2(i) = dlog(tmfp2(i))
        cstop(i) = dlog(cstop(i))
        rstop(i) = dlog(rstop(i))
        awpar(i) = dlog(awpar(i))
com: Skip 1st value because range(1)=0
        if (i.ne.1) range(i) = dlog(range(i))
 30   continue
com: now set 1st range linearly to avoid complications with logs:
      range(1) = (range(3)-range(2))/(energy(3)-energy(2))*
     &             (energy(1)-energy(2))+range(2)
      call spline(energy,elmfp,elmfpa,elmfpb,elmfpc,elmfpd,
     &            0.d0,0.d0,ndata)
      call spline(energy,tmfp1,tmfp1a,tmfp1b,tmfp1c,tmfp1d,
     &            0.d0,0.d0,ndata)
      call spline(energy,tmfp2,tmfp2a,tmfp2b,tmfp2c,tmfp2d,
     &            0.d0,0.d0,ndata)
      call spline(energy,cstop,cstopa,cstopb,cstopc,cstopd,
     &            0.d0,0.d0,ndata)
      call spline(energy,rstop,rstopa,rstopb,rstopc,rstopd,
     &            0.d0,0.d0,ndata)
      call spline(energy,range,rangea,rangeb,rangec,ranged,
     &            0.d0,0.d0,ndata)
      call spline(range,energy,eranga,erangb,erangc,erangd,
     &            0.d0,0.d0,ndata)
      call spline(energy,awpar,awpara,awparb,awparc,awpard,
     &            0.d0,0.d0,ndata)

      read(10,1001) ndata
      do 31  i=1,ndata
        read(10,'(a100)') buffer
   31 continue

      read(10,1001) ndatph
      if (ndatph.gt.mad.or.ndatph.lt.10) then
        write(*,*) 'inimat Error: invalid ndatph: '
        write(*,'(1x,i5)') ndatph
        stop
      endif
      newi = 1
      do 90 i=1,ndatph
        read(10,*) enepho(i),rayl(i),cmpt(i),phot(i),pair(i)
        if (i.eq.1) epinf = enepho(i)
        if (i.eq.ndatph) epsup = enepho(i)
        if (i.ne.1) then
          if (dabs((enepho(i)-enepho(i-1))/enepho(i-1)).lt.eps) then
c           *** If photon table contains an absorption edge,
c             * rise lowest energy to enepho(i):
            newi = i
          endif
        endif
   90 continue
      do 95 i1=newi,ndatph
        i=i1-newi+1
        rayl(i) = dlog(rayl(i1))
        cmpt(i) = dlog(cmpt(i1))
        phot(i) = dlog(phot(i1))
com: Note: 3spline cannot be used below threshold for pair
        if (enepho(i1).gt.1024.d3) then
          pair(i) = dlog(pair(i1))
        else
          ipairs = i
        endif
        enepho(i) = dlog(enepho(i1))
 95   continue
      ndatph = ndatph - newi + 1
      write(*,1010) ndatph
      call spline(enepho,cmpt,cmpta,cmptb,cmptc,cmptd,
     &            0.d0,0.d0,ndatph)
      call spline(enepho,phot,phota,photb,photc,photd,
     &            0.d0,0.d0,ndatph)
com: Arrange pair interpolation to start above threshold:
      ndatpp = ndatph-ipairs
      do 100 i=1,ndatpp
        epairp(i) = enepho(i+ipairs)
        pair(i) = pair(i+ipairs)
 100  continue
      call spline(epairp,pair,paira,pairb,pairc,paird,
     &            0.d0,0.d0,ndatpp)
      end


      real*8 function lambda(e)
c*******************************************************************
c*    Gives elastic mean free path as a function of energy         *
c*                                                                 *
c*    Input:                                                       *
c*      e -> Kinetic energy in eV                                  *
c*    Output:                                                      *
c*      lambda -> mfp in g/cm^2                                    *
c*    Comments:                                                    *
c*      -> inimat() must be called before first call               *
c*      -> 3-spline approx                                         *
c*******************************************************************
      implicit none
      real*8 e
      integer mad,ndata
      parameter (mad=200)
      real*8 elmfp,elmfpa,elmfpb,elmfpc,elmfpd
      real*8 tmfp1,tmfp1a,tmfp1b,tmfp1c,tmfp1d
      real*8 tmfp2,tmfp2a,tmfp2b,tmfp2c,tmfp2d
      real*8 cstop,cstopa,cstopb,cstopc,cstopd
      real*8 rstop,rstopa,rstopb,rstopc,rstopd
      real*8 range,rangea,rangeb,rangec,ranged,
     &             eranga,erangb,erangc,erangd
      real*8 awpar,awpara,awparb,awparc,awpard
      real*8 energy
      common /rawipc/
     &  elmfp(mad),elmfpa(mad),elmfpb(mad),elmfpc(mad),elmfpd(mad),
     &  tmfp1(mad),tmfp1a(mad),tmfp1b(mad),tmfp1c(mad),tmfp1d(mad),
     &  tmfp2(mad),tmfp2a(mad),tmfp2b(mad),tmfp2c(mad),tmfp2d(mad),
     &  cstop(mad),cstopa(mad),cstopb(mad),cstopc(mad),cstopd(mad),
     &  rstop(mad),rstopa(mad),rstopb(mad),rstopc(mad),rstopd(mad),
     &  range(mad),rangea(mad),rangeb(mad),rangec(mad),ranged(mad),
     &             eranga(mad),erangb(mad),erangc(mad),erangd(mad),
     &  awpar(mad),awpara(mad),awparb(mad),awparc(mad),awpard(mad),
     &  energy(mad),ndata
      integer i
      real*8 x

      x = dlog(e)
      call findi(energy,x,ndata,i)
      lambda = dexp(elmfpa(i)+x*(elmfpb(i)+x*(elmfpc(i)+x*elmfpd(i))))
      end


      real*8 function lambd1(e)
c*******************************************************************
c*    Gives 1st transport mean free path as a function of energy   *
c*                                                                 *
c*    Input:                                                       *
c*      e -> Kinetic energy in eV                                  *
c*    Output:                                                      *
c*      lambd1 -> 1st mfp in g/cm^2                                *
c*    Comments:                                                    *
c*      -> inimat() must be called before first call               *
c*      -> 3-spline approx                                         *
c*******************************************************************
      implicit none
      real*8 e
      integer mad,ndata
      parameter (mad=200)
      real*8 elmfp,elmfpa,elmfpb,elmfpc,elmfpd
      real*8 tmfp1,tmfp1a,tmfp1b,tmfp1c,tmfp1d
      real*8 tmfp2,tmfp2a,tmfp2b,tmfp2c,tmfp2d
      real*8 cstop,cstopa,cstopb,cstopc,cstopd
      real*8 rstop,rstopa,rstopb,rstopc,rstopd
      real*8 range,rangea,rangeb,rangec,ranged,
     &             eranga,erangb,erangc,erangd
      real*8 awpar,awpara,awparb,awparc,awpard
      real*8 energy
      common /rawipc/
     &  elmfp(mad),elmfpa(mad),elmfpb(mad),elmfpc(mad),elmfpd(mad),
     &  tmfp1(mad),tmfp1a(mad),tmfp1b(mad),tmfp1c(mad),tmfp1d(mad),
     &  tmfp2(mad),tmfp2a(mad),tmfp2b(mad),tmfp2c(mad),tmfp2d(mad),
     &  cstop(mad),cstopa(mad),cstopb(mad),cstopc(mad),cstopd(mad),
     &  rstop(mad),rstopa(mad),rstopb(mad),rstopc(mad),rstopd(mad),
     &  range(mad),rangea(mad),rangeb(mad),rangec(mad),ranged(mad),
     &             eranga(mad),erangb(mad),erangc(mad),erangd(mad),
     &  awpar(mad),awpara(mad),awparb(mad),awparc(mad),awpard(mad),
     &  energy(mad),ndata
      integer i
      real*8 x

      x = dlog(e)
      call findi(energy,x,ndata,i)
      lambd1 = dexp(tmfp1a(i)+x*(tmfp1b(i)+x*(tmfp1c(i)+x*tmfp1d(i))))
      end


      real*8 function lambd2(e)
c*******************************************************************
c*    Gives 2nd transport mean free path as a function of energy   *
c*                                                                 *
c*    Input:                                                       *
c*      e -> Kinetic energy in eV                                  *
c*    Output:                                                      *
c*      lambd2 -> 2nd mfp in g/cm^2                                *
c*    Comments:                                                    *
c*      -> inimat() must be called before first call               *
c*      -> 3-spline approx                                         *
c*******************************************************************
      implicit none
      real*8 e
      integer mad,ndata
      parameter (mad=200)
      real*8 elmfp,elmfpa,elmfpb,elmfpc,elmfpd
      real*8 tmfp1,tmfp1a,tmfp1b,tmfp1c,tmfp1d
      real*8 tmfp2,tmfp2a,tmfp2b,tmfp2c,tmfp2d
      real*8 cstop,cstopa,cstopb,cstopc,cstopd
      real*8 rstop,rstopa,rstopb,rstopc,rstopd
      real*8 range,rangea,rangeb,rangec,ranged,
     &             eranga,erangb,erangc,erangd
      real*8 awpar,awpara,awparb,awparc,awpard
      real*8 energy
      common /rawipc/
     &  elmfp(mad),elmfpa(mad),elmfpb(mad),elmfpc(mad),elmfpd(mad),
     &  tmfp1(mad),tmfp1a(mad),tmfp1b(mad),tmfp1c(mad),tmfp1d(mad),
     &  tmfp2(mad),tmfp2a(mad),tmfp2b(mad),tmfp2c(mad),tmfp2d(mad),
     &  cstop(mad),cstopa(mad),cstopb(mad),cstopc(mad),cstopd(mad),
     &  rstop(mad),rstopa(mad),rstopb(mad),rstopc(mad),rstopd(mad),
     &  range(mad),rangea(mad),rangeb(mad),rangec(mad),ranged(mad),
     &             eranga(mad),erangb(mad),erangc(mad),erangd(mad),
     &  awpar(mad),awpara(mad),awparb(mad),awparc(mad),awpard(mad),
     &  energy(mad),ndata
      integer i
      real*8 x

      x = dlog(e)
      call findi(energy,x,ndata,i)
      lambd2 = dexp(tmfp2a(i)+x*(tmfp2b(i)+x*(tmfp2c(i)+x*tmfp2d(i))))
      end


      real*8 function cstpow(e)
c*******************************************************************
c*    Collision stopping power as a funtion of energy              *
c*                                                                 *
c*    Input:                                                       *
c*      e -> Kinetic energy in eV                                  *
c*    Output:                                                      *
c*      stopping power in eV*cm^2/g                                *
c*    Comments:                                                    *
c*      -> inimat() must be called before first call               *
c*      -> 3-spline approx                                         *
c*******************************************************************
      implicit none
      real*8 e
      integer mad,ndata
      parameter (mad=200)
      real*8 elmfp,elmfpa,elmfpb,elmfpc,elmfpd
      real*8 tmfp1,tmfp1a,tmfp1b,tmfp1c,tmfp1d
      real*8 tmfp2,tmfp2a,tmfp2b,tmfp2c,tmfp2d
      real*8 cstop,cstopa,cstopb,cstopc,cstopd
      real*8 rstop,rstopa,rstopb,rstopc,rstopd
      real*8 range,rangea,rangeb,rangec,ranged,
     &             eranga,erangb,erangc,erangd
      real*8 awpar,awpara,awparb,awparc,awpard
      real*8 energy
      common /rawipc/
     &  elmfp(mad),elmfpa(mad),elmfpb(mad),elmfpc(mad),elmfpd(mad),
     &  tmfp1(mad),tmfp1a(mad),tmfp1b(mad),tmfp1c(mad),tmfp1d(mad),
     &  tmfp2(mad),tmfp2a(mad),tmfp2b(mad),tmfp2c(mad),tmfp2d(mad),
     &  cstop(mad),cstopa(mad),cstopb(mad),cstopc(mad),cstopd(mad),
     &  rstop(mad),rstopa(mad),rstopb(mad),rstopc(mad),rstopd(mad),
     &  range(mad),rangea(mad),rangeb(mad),rangec(mad),ranged(mad),
     &             eranga(mad),erangb(mad),erangc(mad),erangd(mad),
     &  awpar(mad),awpara(mad),awparb(mad),awparc(mad),awpard(mad),
     &  energy(mad),ndata
      integer i
      real*8 x

      x = dlog(e)
      call findi(energy,x,ndata,i)
      cstpow = dexp(cstopa(i)+x*(cstopb(i)+x*(cstopc(i)+x*cstopd(i))))
      end


      real*8 function rstpow(e)
c*******************************************************************
c*    Gives radiative (bremss) stopping power as energy function   *
c*                                                                 *
c*    Input:                                                       *
c*      e -> Kinetic energy in eV                                  *
c*    Output:                                                      *
c*      Stopping power in eV*cm^2/g                                *
c*    Comments:                                                    *
c*      -> inimat() must be called before first call               *
c*      -> 3-spline approx                                         *
c*******************************************************************
      implicit none
      real*8 e
      integer mad,ndata
      parameter (mad=200)
      real*8 elmfp,elmfpa,elmfpb,elmfpc,elmfpd
      real*8 tmfp1,tmfp1a,tmfp1b,tmfp1c,tmfp1d
      real*8 tmfp2,tmfp2a,tmfp2b,tmfp2c,tmfp2d
      real*8 cstop,cstopa,cstopb,cstopc,cstopd
      real*8 rstop,rstopa,rstopb,rstopc,rstopd
      real*8 range,rangea,rangeb,rangec,ranged,
     &             eranga,erangb,erangc,erangd
      real*8 awpar,awpara,awparb,awparc,awpard
      real*8 energy
      common /rawipc/
     &  elmfp(mad),elmfpa(mad),elmfpb(mad),elmfpc(mad),elmfpd(mad),
     &  tmfp1(mad),tmfp1a(mad),tmfp1b(mad),tmfp1c(mad),tmfp1d(mad),
     &  tmfp2(mad),tmfp2a(mad),tmfp2b(mad),tmfp2c(mad),tmfp2d(mad),
     &  cstop(mad),cstopa(mad),cstopb(mad),cstopc(mad),cstopd(mad),
     &  rstop(mad),rstopa(mad),rstopb(mad),rstopc(mad),rstopd(mad),
     &  range(mad),rangea(mad),rangeb(mad),rangec(mad),ranged(mad),
     &             eranga(mad),erangb(mad),erangc(mad),erangd(mad),
     &  awpar(mad),awpara(mad),awparb(mad),awparc(mad),awpard(mad),
     &  energy(mad),ndata
      integer i
      real*8 x

      x = dlog(e)
      call findi(energy,x,ndata,i)
      rstpow = dexp(rstopa(i)+x*(rstopb(i)+x*(rstopc(i)+x*rstopd(i))))
      end


      real*8 function rang(e)
c*******************************************************************
c*    Range as a function of the energy                            *
c*                                                                 *
c*    Input:                                                       *
c*      e -> Kinetic energy in eV                                  *
c*    Output:                                                      *
c*      range in g/cm^2                                            *
c*    Comments:                                                    *
c*      -> inimat() must be called before first call               *
c*      -> it is assumed that the range=0 for the 1st energy found *
c*         in the input data file --see inimat()--                 *
c*      -> 3-spline approx                                         *
c*******************************************************************
      implicit none
      real*8 e
      integer mad,ndata
      parameter (mad=200)
      real*8 elmfp,elmfpa,elmfpb,elmfpc,elmfpd
      real*8 tmfp1,tmfp1a,tmfp1b,tmfp1c,tmfp1d
      real*8 tmfp2,tmfp2a,tmfp2b,tmfp2c,tmfp2d
      real*8 cstop,cstopa,cstopb,cstopc,cstopd
      real*8 rstop,rstopa,rstopb,rstopc,rstopd
      real*8 range,rangea,rangeb,rangec,ranged,
     &             eranga,erangb,erangc,erangd
      real*8 awpar,awpara,awparb,awparc,awpard
      real*8 energy
      common /rawipc/
     &  elmfp(mad),elmfpa(mad),elmfpb(mad),elmfpc(mad),elmfpd(mad),
     &  tmfp1(mad),tmfp1a(mad),tmfp1b(mad),tmfp1c(mad),tmfp1d(mad),
     &  tmfp2(mad),tmfp2a(mad),tmfp2b(mad),tmfp2c(mad),tmfp2d(mad),
     &  cstop(mad),cstopa(mad),cstopb(mad),cstopc(mad),cstopd(mad),
     &  rstop(mad),rstopa(mad),rstopb(mad),rstopc(mad),rstopd(mad),
     &  range(mad),rangea(mad),rangeb(mad),rangec(mad),ranged(mad),
     &             eranga(mad),erangb(mad),erangc(mad),erangd(mad),
     &  awpar(mad),awpara(mad),awparb(mad),awparc(mad),awpard(mad),
     &  energy(mad),ndata
      integer i
      real*8 x,zero
      parameter (zero=1.d-30)

      if (e.le.0.d0) then
        rang = zero
        return
      endif
      x = dlog(e)
      if (x.le.energy(1)) then
        rang = zero
        return
      endif
      call findi(energy,x,ndata,i)
      rang=dexp(rangea(i)+x*(rangeb(i)+x*(rangec(i)+x*ranged(i))))
      end


      real*8 function energ(rang)
c*******************************************************************
c*    Kinetic energy as a function of the range                    *
c*                                                                 *
c*    Input:                                                       *
c*      rang -> range in g/cm^2                                    *
c*    Output:                                                      *
c*      Kinetic energy in eV                                       *
c*    Comments:                                                    *
c*      -> inimat() must be called before first call               *
c*      -> it is assumed that range=0 for the 1st energy found     *
c*         in the input data file --see inimat()                   *
c*      -> 3-spline approx                                         *
c*******************************************************************
      implicit none
      real*8 rang
      integer mad,ndata
      parameter (mad=200)
      real*8 elmfp,elmfpa,elmfpb,elmfpc,elmfpd
      real*8 tmfp1,tmfp1a,tmfp1b,tmfp1c,tmfp1d
      real*8 tmfp2,tmfp2a,tmfp2b,tmfp2c,tmfp2d
      real*8 cstop,cstopa,cstopb,cstopc,cstopd
      real*8 rstop,rstopa,rstopb,rstopc,rstopd
      real*8 range,rangea,rangeb,rangec,ranged,
     &             eranga,erangb,erangc,erangd
      real*8 awpar,awpara,awparb,awparc,awpard
      real*8 energy
      common /rawipc/
     &  elmfp(mad),elmfpa(mad),elmfpb(mad),elmfpc(mad),elmfpd(mad),
     &  tmfp1(mad),tmfp1a(mad),tmfp1b(mad),tmfp1c(mad),tmfp1d(mad),
     &  tmfp2(mad),tmfp2a(mad),tmfp2b(mad),tmfp2c(mad),tmfp2d(mad),
     &  cstop(mad),cstopa(mad),cstopb(mad),cstopc(mad),cstopd(mad),
     &  rstop(mad),rstopa(mad),rstopb(mad),rstopc(mad),rstopd(mad),
     &  range(mad),rangea(mad),rangeb(mad),rangec(mad),ranged(mad),
     &             eranga(mad),erangb(mad),erangc(mad),erangd(mad),
     &  awpar(mad),awpara(mad),awparb(mad),awparc(mad),awpard(mad),
     &  energy(mad),ndata
      integer i
      real*8 x

      if (rang.le.0.d0) then
        energ = dexp(energy(1))
        return
      endif
      x = dlog(rang)
      if (x.le.range(1)) then
        energ = dexp(energy(1))
        return
      endif
      call findi(range,x,ndata,i)
      energ=dexp(eranga(i)+x*(erangb(i)+x*(erangc(i)+x*erangd(i))))
      end


      real*8 function awsolv(G1,accu)
c*******************************************************************
c*    Calculates the screening parameter of a screened Rutherford  *
c*    DCS by finding the value that matches the 1st transport      *
c*    coefficient                                                  *
c*                                                                 *
c*    Input:                                                       *
c*      G1 -> 1-<cos(theta)> --1st transport coef--                *
c*      accu -> fractional accuracy required (to match G1)         *
c*    Output:                                                      *
c*      awsolv -> screening parameter aw in                        *
c*                screened Rutherford DCS ~ (1+2*aw-mu)^-2         *
c*******************************************************************
      implicit none
      real*8 G1,accu
      real*8 aw,faw,dfaw,log1aw,halfG1,invhaf

      halfG1 = G1/2.d0
      invhaf = 1.d0/halfG1

com:  assure that aw is below the value sought
      aw = halfG1
 30   continue
      if (aw*((1.d0+aw)*dlog(1.d0+1.d0/aw)-1.d0).gt.halfG1) then
        aw = aw/10.d0
        goto 30
      endif

com:  apply Newton's method
 10   continue
        log1aw = dlog(1.d0+1.d0/aw)
        faw = aw*((1.d0+aw)*log1aw-1.d0)
        if (dabs(faw*invhaf-1.d0).lt.accu) goto 20
        faw = faw-halfG1
        dfaw = -2.d0+(1.d0+2.d0*aw)*log1aw
        aw = aw-faw/dfaw
      goto 10
 20   continue
      awsolv = aw
      end


      real*8 function awf(e)
c*******************************************************************
c*    Screening parameter for the screened Rutherford DCS          *
c*    ~ (1+2*aw-mu)^-2  as a function of the kinetic energy        *
c*                                                                 *
c*    Input:                                                       *
c*      e -> kinetic energy in eV                                  *
c*    Comments:                                                    *
c*      -> inimat() must be called before first call               *
c*      -> 3-spline approx                                         *
c*******************************************************************
      implicit none
      real*8 e
      integer mad,ndata
      parameter (mad=200)
      real*8 elmfp,elmfpa,elmfpb,elmfpc,elmfpd
      real*8 tmfp1,tmfp1a,tmfp1b,tmfp1c,tmfp1d
      real*8 tmfp2,tmfp2a,tmfp2b,tmfp2c,tmfp2d
      real*8 cstop,cstopa,cstopb,cstopc,cstopd
      real*8 rstop,rstopa,rstopb,rstopc,rstopd
      real*8 range,rangea,rangeb,rangec,ranged,
     &             eranga,erangb,erangc,erangd
      real*8 awpar,awpara,awparb,awparc,awpard
      real*8 energy
      common /rawipc/
     &  elmfp(mad),elmfpa(mad),elmfpb(mad),elmfpc(mad),elmfpd(mad),
     &  tmfp1(mad),tmfp1a(mad),tmfp1b(mad),tmfp1c(mad),tmfp1d(mad),
     &  tmfp2(mad),tmfp2a(mad),tmfp2b(mad),tmfp2c(mad),tmfp2d(mad),
     &  cstop(mad),cstopa(mad),cstopb(mad),cstopc(mad),cstopd(mad),
     &  rstop(mad),rstopa(mad),rstopb(mad),rstopc(mad),rstopd(mad),
     &  range(mad),rangea(mad),rangeb(mad),rangec(mad),ranged(mad),
     &             eranga(mad),erangb(mad),erangc(mad),erangd(mad),
     &  awpar(mad),awpara(mad),awparb(mad),awparc(mad),awpard(mad),
     &  energy(mad),ndata
      integer i
      real*8 x

      x = dlog(e)
      call findi(energy,x,ndata,i)
      awf=dexp(awpara(i)+x*(awparb(i)+x*(awparc(i)+x*awpard(i))))
      end


      real*8 function icompt(e)
c*******************************************************************
c*    Inverse of the Compton mean free path as a function of       *
c*    the kinetic energy                                           *
c*                                                                 *
c*    Input:                                                       *
c*      e -> in eV                                                 *
c*    Output:                                                      *
c*      -> lambda^-1 in cm^2/g                                     *
c*    Comments:                                                    *
c*      -> inimat() must be called before first call               *
c*      -> 3-spline approx                                         *
c*******************************************************************
      implicit none
      real*8 e
      integer mad
      parameter (mad=200)
      integer ndatph,ndatpp
      real*8 enepho,rayl,cmpt,phot,epairp,pair
      common /tabpho/
     &  enepho(mad),rayl(mad),cmpt(mad),phot(mad),
     &  epairp(mad),pair(mad),ndatph,ndatpp
      real*8
     &  rayla,raylb,raylc,rayld,cmpta,cmptb,cmptc,cmptd,
     &  phota,photb,photc,photd,paira,pairb,pairc,paird
      common /ippho/
     &  rayla(mad),raylb(mad),raylc(mad),rayld(mad),
     &  cmpta(mad),cmptb(mad),cmptc(mad),cmptd(mad),
     &  phota(mad),photb(mad),photc(mad),photd(mad),
     &  paira(mad),pairb(mad),pairc(mad),paird(mad)
      integer i
      real*8 x

      x = dlog(e)
      call findi(enepho,x,ndatph,i)
      icompt=dexp(cmpta(i)+x*(cmptb(i)+x*(cmptc(i)+x*cmptd(i))))
      end


      real*8 function iphote(e)
c*******************************************************************
c*    Inverse of the photoelectric absorption mean free path as    *
c*    a function of the kinetic energy                             *
c*                                                                 *
c*    Input:                                                       *
c*      e -> in eV                                                 *
c*    Output:                                                      *
c*      -> lambda^-1 in cm^2/g                                     *
c*    Comments:                                                    *
c*      -> inimat() must be called before first call               *
c*      -> 3-spline approx                                         *
c*******************************************************************
      implicit none
      real*8 e
      integer mad
      parameter (mad=200)
      integer ndatph,ndatpp
      real*8 enepho,rayl,cmpt,phot,epairp,pair
      common /tabpho/
     &  enepho(mad),rayl(mad),cmpt(mad),phot(mad),
     &  epairp(mad),pair(mad),ndatph,ndatpp
      real*8
     &  rayla,raylb,raylc,rayld,cmpta,cmptb,cmptc,cmptd,
     &  phota,photb,photc,photd,paira,pairb,pairc,paird
      common /ippho/
     &  rayla(mad),raylb(mad),raylc(mad),rayld(mad),
     &  cmpta(mad),cmptb(mad),cmptc(mad),cmptd(mad),
     &  phota(mad),photb(mad),photc(mad),photd(mad),
     &  paira(mad),pairb(mad),pairc(mad),paird(mad)
      integer i
      real*8 x

      x = dlog(e)
      call findi(enepho,x,ndatph,i)
      iphote=dexp(phota(i)+x*(photb(i)+x*(photc(i)+x*photd(i))))
      end


      real*8 function ipairp(e)
c*******************************************************************
c*    Inverse of the pair production mean free path as a function  *
c*    of the kinetic energy                                        *
c*                                                                 *
c*    Input:                                                       *
c*      e -> in eV                                                 *
c*    Output:                                                      *
c*      -> lambda^-1 in cm^2/g                                     *
c*    Comments:                                                    *
c*      -> inimat() must be called before first call               *
c*      -> 3-spline approx                                         *
c*******************************************************************
      implicit none
      real*8 e
      integer mad
      parameter (mad=200)
      integer ndatph,ndatpp
      real*8 enepho,rayl,cmpt,phot,epairp,pair
      common /tabpho/
     &  enepho(mad),rayl(mad),cmpt(mad),phot(mad),
     &  epairp(mad),pair(mad),ndatph,ndatpp
      real*8
     &  rayla,raylb,raylc,rayld,cmpta,cmptb,cmptc,cmptd,
     &  phota,photb,photc,photd,paira,pairb,pairc,paird
      common /ippho/
     &  rayla(mad),raylb(mad),raylc(mad),rayld(mad),
     &  cmpta(mad),cmptb(mad),cmptc(mad),cmptd(mad),
     &  phota(mad),photb(mad),photc(mad),photd(mad),
     &  paira(mad),pairb(mad),pairc(mad),paird(mad)
      integer i
      real*8 x

com: No pair production below the threshold:
      if (e.lt.1024.d3) then
        ipairp = 0.d0
        return
      endif
      x = dlog(e)
      call findi(epairp,x,ndatpp,i)
      ipairp=dexp(paira(i)+x*(pairb(i)+x*(pairc(i)+x*paird(i))))
      end


      block data atodat
c*******************************************************************
c*    Atomic data shared by other routines                         *
c*******************************************************************
      implicit none
      real*8 atm
      common /catmas/ atm(99)
      real*8 mee
      common /cmexce/ mee(99)
      real*8 rs,eta
      COMMON/CPAIR/RS(100),ETA(100)

com: Atomic weights in amu...
      data atm  /1.0079d0,4.0026d0,6.9410d0,9.0122d0,1.0811d1,
     1  1.2011d1,1.4007d1,1.5999d1,1.8998d1,2.0179d1,2.2990d1,
     2  2.4305d1,2.6982d1,2.8086d1,3.0974d1,3.2066d1,3.5453d1,
     3  3.9948d1,3.9098d1,4.0078d1,4.4956d1,4.7880d1,5.0942d1,
     4  5.1996d1,5.4938d1,5.5847d1,5.8933d1,5.8690d1,6.3546d1,
     5  6.5390d1,6.9723d1,7.2610d1,7.4922d1,7.8960d1,7.9904d1,
     6  8.3800d1,8.5468d1,8.7620d1,8.8906d1,9.1224d1,9.2906d1,
     7  9.5940d1,9.7907d1,1.0107d2,1.0291d2,1.0642d2,1.0787d2,
     8  1.1241d2,1.1482d2,1.1871d2,1.2175d2,1.2760d2,1.2690d2,
     9  1.3129d2,1.3291d2,1.3733d2,1.3891d2,1.4012d2,1.4091d2,
     a  1.4424d2,1.4491d2,1.5036d2,1.5196d2,1.5725d2,1.5893d2,
     b  1.6250d2,1.6493d2,1.6726d2,1.6893d2,1.7304d2,1.7497d2,
     c  1.7849d2,1.8095d2,1.8385d2,1.8621d2,1.9020d2,1.9222d2,
     d  1.9508d2,1.9697d2,2.0059d2,2.0438d2,2.0720d2,2.0898d2,
     e  2.0898d2,2.0999d2,2.2202d2,2.2302d2,2.2603d2,2.2703d2,
     f  2.3204d2,2.3104d2,2.3803d2,2.3705d2,2.3905d2,2.4306d2,
     g  2.4707d2,2.4707d2,2.5108d2,2.5208d2/

      data mee / 19.2d0, 41.8d0, 40.0d0, 63.7d0, 76.0d0, 81.0d0,
     1   82.0d0, 95.0d0,115.0d0,137.0d0,149.0d0,156.0d0,166.0d0,
     2  173.0d0,173.0d0,180.0d0,174.0d0,188.0d0,190.0d0,191.0d0,
     3  216.0d0,233.0d0,245.0d0,257.0d0,272.0d0,286.0d0,297.0d0,
     4  311.0d0,322.0d0,330.0d0,334.0d0,350.0d0,347.0d0,348.0d0,
     5  343.0d0,352.0d0,363.0d0,366.0d0,379.0d0,393.0d0,417.0d0,
     6  424.0d0,428.0d0,441.0d0,449.0d0,470.0d0,470.0d0,469.0d0,
     7  488.0d0,488.0d0,487.0d0,485.0d0,491.0d0,482.0d0,488.0d0,
     8  491.0d0,501.0d0,523.0d0,535.0d0,546.0d0,560.0d0,574.0d0,
     9  580.0d0,591.0d0,614.0d0,628.0d0,650.0d0,658.0d0,674.0d0,
     a  684.0d0,694.0d0,705.0d0,718.0d0,727.0d0,736.0d0,746.0d0,
     b  757.0d0,790.0d0,790.0d0,800.0d0,810.0d0,823.0d0,823.0d0,
     c  830.0d0,825.0d0,794.0d0,827.0d0,826.0d0,841.0d0,847.0d0,
     d  878.0d0,890.0d0,902.0d0,921.0d0,934.0d0,939.0d0,952.0d0,
     e  966.0d0,980.0d0/
c  ************  PAIR PRODUCTION CROSS SECTION PARAMETERS.
c  ****  SCREENING PARAMETER.
      DATA RS    /1.2281D2,7.3167D1,6.9228D1,6.7301D1,6.4696D1,
     1   6.1228D1,5.7524D1,5.4033D1,5.0787D1,4.7851D1,4.6373D1,
     2   4.5401D1,4.4503D1,4.3815D1,4.3074D1,4.2321D1,4.1586D1,
     3   4.0953D1,4.0524D1,4.0256D1,3.9756D1,3.9144D1,3.8462D1,
     4   3.7778D1,3.7174D1,3.6663D1,3.5986D1,3.5317D1,3.4688D1,
     5   3.4197D1,3.3786D1,3.3422D1,3.3068D1,3.2740D1,3.2438D1,
     6   3.2143D1,3.1884D1,3.1622D1,3.1438D1,3.1142D1,3.0950D1,
     7   3.0758D1,3.0561D1,3.0285D1,3.0097D1,2.9832D1,2.9581D1,
     8   2.9411D1,2.9247D1,2.9085D1,2.8930D1,2.8721D1,2.8580D1,
     9   2.8442D1,2.8312D1,2.8139D1,2.7973D1,2.7819D1,2.7675D1,
     A   2.7496D1,2.7285D1,2.7093D1,2.6911D1,2.6705D1,2.6516D1,
     B   2.6304D1,2.6108D1,2.5929D1,2.5730D1,2.5577D1,2.5403D1,
     C   2.5245D1,2.5100D1,2.4941D1,2.4790D1,2.4655D1,2.4506D1,
     D   2.4391D1,2.4262D1,2.4145D1,2.4039D1,2.3922D1,2.3813D1,
     E   2.3712D1,2.3621D1,2.3523D1,2.3430D1,2.3331D1,2.3238D1,
     F   2.3139D1,2.3048D1,2.2967D1,2.2833D1,2.2694D1,2.2624D1,
     G   2.2545D1,2.2446D1,2.2358D1,2.2264D1,2.2178D1/
c  ****  ASYMPTOTIC TRIPLET CONTRIBUTION.
      DATA ETA   /1.1570D0,1.1690D0,1.2190D0,1.2010D0,1.1890D0,
     1   1.1740D0,1.1760D0,1.1690D0,1.1630D0,1.1570D0,1.1740D0,
     2   1.1830D0,1.1860D0,1.1840D0,1.1800D0,1.1780D0,1.1750D0,
     3   1.1700D0,1.1800D0,1.1870D0,1.1840D0,1.1800D0,1.1770D0,
     4   1.1660D0,1.1690D0,1.1660D0,1.1640D0,1.1620D0,1.1540D0,
     5   1.1560D0,1.1570D0,1.1580D0,1.1570D0,1.1580D0,1.1580D0,
     6   1.1580D0,1.1660D0,1.1730D0,1.1740D0,1.1750D0,1.1700D0,
     7   1.1690D0,1.1720D0,1.1690D0,1.1680D0,1.1640D0,1.1670D0,
     8   1.1700D0,1.1720D0,1.1740D0,1.1750D0,1.1780D0,1.1790D0,
     9   1.1800D0,1.1870D0,1.1940D0,1.1970D0,1.1960D0,1.1940D0,
     A   1.1940D0,1.1940D0,1.1940D0,1.1940D0,1.1960D0,1.1970D0,
     B   1.1960D0,1.1970D0,1.1970D0,1.1980D0,1.1980D0,1.2000D0,
     C   1.2010D0,1.2020D0,1.2040D0,1.2050D0,1.2060D0,1.2080D0,
     D   1.2070D0,1.2080D0,1.2120D0,1.2150D0,1.2180D0,1.2210D0,
     E   1.2240D0,1.2270D0,1.2300D0,1.2370D0,1.2430D0,1.2470D0,
     F   1.2500D0,1.2510D0,1.2520D0,1.2550D0,1.2560D0,1.2570D0,
     G   1.2590D0,1.2620D0,1.2620D0,1.2650D0,1.2660D0/
      end



c*******************************************************************
c*******************************************************************
c*   Combined functions routines                                   *
c*   Intended to provide interpolated combined quantities such as  *
c*   total (elastic _and_ inelastic) mean free path, etc           *
c*******************************************************************
c*******************************************************************


      subroutine initot()
c*******************************************************************
c*    Initializes the interpolation of combined quantities         *
c*                                                                 *
c*    Comments:                                                    *
c*      -> inimat() & iniine() must be called before, in this order*
c*      -> notice that the call to iniine() should set a Moller    *
c*         cutoff to account for _all_ the scattering power,       *
c*         typically Wmol = I_excite                               *
c*******************************************************************
      implicit none
      real*8 atno,atno2,atno21,mass,dens,excite
      common /mater/ atno,atno2,atno21,mass,dens,excite
      integer mad,ndata
      parameter (mad=200)
      real*8 elmfp,elmfpa,elmfpb,elmfpc,elmfpd
      real*8 tmfp1,tmfp1a,tmfp1b,tmfp1c,tmfp1d
      real*8 tmfp2,tmfp2a,tmfp2b,tmfp2c,tmfp2d
      real*8 cstop,cstopa,cstopb,cstopc,cstopd
      real*8 rstop,rstopa,rstopb,rstopc,rstopd
      real*8 range,rangea,rangeb,rangec,ranged,
     &             eranga,erangb,erangc,erangd
      real*8 awpar,awpara,awparb,awparc,awpard
      real*8 energy
      common /rawipc/
     &  elmfp(mad),elmfpa(mad),elmfpb(mad),elmfpc(mad),elmfpd(mad),
     &  tmfp1(mad),tmfp1a(mad),tmfp1b(mad),tmfp1c(mad),tmfp1d(mad),
     &  tmfp2(mad),tmfp2a(mad),tmfp2b(mad),tmfp2c(mad),tmfp2d(mad),
     &  cstop(mad),cstopa(mad),cstopb(mad),cstopc(mad),cstopd(mad),
     &  rstop(mad),rstopa(mad),rstopb(mad),rstopc(mad),rstopd(mad),
     &  range(mad),rangea(mad),rangeb(mad),rangec(mad),ranged(mad),
     &             eranga(mad),erangb(mad),erangc(mad),erangd(mad),
     &  awpar(mad),awpara(mad),awparb(mad),awparc(mad),awpard(mad),
     &  energy(mad),ndata
      integer ndatt
      real*8 ilact,ilacta,ilactb,ilactc,ilactd
      real*8 scptt,scptta,scpttb,scpttc,scpttd
      real*8 awctt,awctta,awcttb,awcttc,awcttd
      real*8 enetot
      common /totalc/
     &  ilact(mad),ilacta(mad),ilactb(mad),ilactc(mad),ilactd(mad),
     &  scptt(mad),scptta(mad),scpttb(mad),scpttc(mad),scpttd(mad),
     &  awctt(mad),awctta(mad),awcttb(mad),awcttc(mad),awcttd(mad),
     &  enetot(mad),ndatt
      real*8 wcion,wcbre
      common /cutoff/ wcion,wcbre

      integer i
      real*8 awsolv,e,G1,lambda,lambd1,accu,kcmax
      parameter (accu=1.d-10,kcmax=0.4999999999d0)

      ndatt = ndata
      do 10 i=1,ndata
com: preparing interpolation tables...
        enetot(i) = energy(i)
        e = dexp(energy(i))
        ilact(i) = (atno21/atno2)/lambda(e)
        scptt(i) = (atno21/atno2)/lambd1(e)
        G1 = scptt(i)/ilact(i)
        awctt(i) = awsolv(G1,accu)
        ilact(i) = dlog(ilact(i))
        scptt(i) = dlog(scptt(i))
        awctt(i) = dlog(awctt(i))
 10   continue
 20   continue

      call spline(enetot,ilact,ilacta,ilactb,ilactc,ilactd,
     &            0.d0,0.d0,ndatt)
      call spline(enetot,scptt,scptta,scpttb,scpttc,scpttd,
     &            0.d0,0.d0,ndatt)
      call spline(enetot,awctt,awctta,awcttb,awcttc,awcttd,
     &            0.d0,0.d0,ndatt)
      end


      real*8 function ilamtt(e)
c*******************************************************************
c*    Inverse 'elastic' MFP as function of kinetic energy,         *
c*    including the inelastic factor Z(Z+1)                        *
c*                                                                 *
c*    Input:                                                       *
c*      e -> kinetic energy in eV                                  *
c*    Output:                                                      *
c*      1/lambda in cm^2/g                                         *
c*    Comments:                                                    *
c*      -> initot() must be called before first call               *
c*      -> 3-spline approx                                         *
c*      -> Uses the Z(Z+1) aprox.                                  *
c*******************************************************************
      implicit none
      real*8 e
      integer mad
      parameter (mad=200)
      integer ndatt
      real*8 ilact,ilacta,ilactb,ilactc,ilactd
      real*8 scptt,scptta,scpttb,scpttc,scpttd
      real*8 awctt,awctta,awcttb,awcttc,awcttd
      real*8 enetot
      common /totalc/
     &  ilact(mad),ilacta(mad),ilactb(mad),ilactc(mad),ilactd(mad),
     &  scptt(mad),scptta(mad),scpttb(mad),scpttc(mad),scpttd(mad),
     &  awctt(mad),awctta(mad),awcttb(mad),awcttc(mad),awcttd(mad),
     &  enetot(mad),ndatt
      integer i
      real*8 x

      x = dlog(e)
      call findi(enetot,x,ndatt,i)
      ilamtt=dexp(ilacta(i)+x*(ilactb(i)+x*(ilactc(i)+x*ilactd(i))))
      end


      real*8 function scptot(e)
c*******************************************************************
c*    Inverse 1st transport mean free path as a function of the    *
c*    kinetic energy, including inelastic scattering               *
c*                                                                 *
c*    Input:                                                       *
c*      e -> kinetic energy in eV                                  *
c*    Output:                                                      *
c*      Inverse 1st transport mean free path in cm^2/g             *
c*    Comments:                                                    *
c*      -> initot() must be called before first call               *
c*      -> 3-spline approx                                         *
c*      -> Uses the Z(Z+1) aprox.                                  *
c*******************************************************************
      implicit none
      real*8 e
      integer mad
      parameter (mad=200)
      integer ndatt
      real*8 ilact,ilacta,ilactb,ilactc,ilactd
      real*8 scptt,scptta,scpttb,scpttc,scpttd
      real*8 awctt,awctta,awcttb,awcttc,awcttd
      real*8 enetot
      common /totalc/
     &  ilact(mad),ilacta(mad),ilactb(mad),ilactc(mad),ilactd(mad),
     &  scptt(mad),scptta(mad),scpttb(mad),scpttc(mad),scpttd(mad),
     &  awctt(mad),awctta(mad),awcttb(mad),awcttc(mad),awcttd(mad),
     &  enetot(mad),ndatt
      integer i
      real*8 x

      x = dlog(e)
      call findi(enetot,x,ndatt,i)
      scptot=dexp(scptta(i)+x*(scpttb(i)+x*(scpttc(i)+x*scpttd(i))))
      end


      real*8 function awtot(e)
c*******************************************************************
c*    Screening parameter for the screened Rutherford DCS          *
c*    ~ (1+2*aw-mu)^-2  as a function of the kinetic energy,       *
c*    taking into account combined scattering data                 *
c*                                                                 *
c*    Input:                                                       *
c*      e -> kinetic energy in eV                                  *
c*    Comments:                                                    *
c*      -> initot() must be called before first call               *
c*      -> 3-spline approx                                         *
c*******************************************************************
      implicit none
      real*8 e
      integer mad
      parameter (mad=200)
      integer ndatt
      real*8 ilact,ilacta,ilactb,ilactc,ilactd
      real*8 scptt,scptta,scpttb,scpttc,scpttd
      real*8 awctt,awctta,awcttb,awcttc,awcttd
      real*8 enetot
      common /totalc/
     &  ilact(mad),ilacta(mad),ilactb(mad),ilactc(mad),ilactd(mad),
     &  scptt(mad),scptta(mad),scpttb(mad),scpttc(mad),scpttd(mad),
     &  awctt(mad),awctta(mad),awcttb(mad),awcttc(mad),awcttd(mad),
     &  enetot(mad),ndatt
      integer i
      real*8 x

      x = dlog(e)
      call findi(enetot,x,ndatt,i)
      awtot = dexp(awctta(i)+x*(awcttb(i)+x*(awcttc(i)+x*awcttd(i))))
      end


c* end of file *****************************************************

