C
C    M    M    AA   TTTTTT  EEEEEE  RRRRR   IIII    AA    L
C    MM  MM   A  A    TT    E       R    R   II    A  A   L
C    M MM M  A    A   TT    E       R    R   II   A    A  L
C    M    M  AAAAAA   TT    EEEE    RRRRR    II   AAAAAA  L
C    M    M  A    A   TT    E       R   R    II   A    A  L
C    M    M  A    A   TT    EEEEEE  R    R  IIII  A    A  LLLLLL
C
C                                           (VERSION 2000.08.28).
C
C    THIS PROGRAM CREATES THE MATERIAL DEFINITION FILE, THE INPUT
C  DATA FILE FOR PENELOPE. A MATERIAL IS COMPLETELY CHARACTERIZED
C  BY ITS CHEMICAL COMPOSITION, I.E. ELEMENTS PRESENT AND NUMBER
C  OF ATOMS OF EACH ELEMENT IN A MOLECULE (=STOICHIOMETRIC
C  INDEX), MASS DENSITY AND MEAN EXCITATION ENERGY. ALLOYS AND
C  MIXTURES ARE TREATED AS COMPOUNDS, WITH STOICHIOMETRIC INDICES
C  EQUAL OR PROPORTIONAL TO THE PERCENT NUMBER OF ATOMS OF EACH
C  ELEMENT. INFORMATION ABOUT THE MATERIAL IS SUPPLIED BY THE
C  USER FROM THE KEYBOARD, FOLLOWING THE PROMPTS FROM 'MATERIAL',
C  OR READ FROM THE 'COMPDATA.TAB' FILE, WHICH CONTAINS DATA FOR
C  279 DIFFERENT MATERIALS. IN THE CASE OF COMPOUNDS, 'MOLECULAR'
C  CROSS SECTIONS ARE OBTAINED BY MEANS OF THE ADDITIVITY RULE,
C  I.E. AS THE SUM OF THE ATOMIC CROSS SECTIONS.
C
C    THE DATABASE CONSISTS OF 187 ASCII FILES WITH MATERIAL
C  COMPOSITION AND PARTICLE INTERACTION DATA. THE NEEDED DATA
C  FILES ARE THE FOLLOWING:
C  *'COMPDATA.TAB': COMPOSITION DATA FOR 279 DIFFERENT MATERIALS.
C  *'IDLIST.TAB': LIST OF MATERIALS INCLUDED IN THE COMPDATA.TAB
C       FILE, WITH IDENTIFICATION NUMBERS.
C  *92 FILES NAMED 'PENEPZZ.TAB' WITH ZZ=ATOMIC NUMBER (01-92).
C       THESE FILES CONTAIN ELECTRON AND POSITRON INTERACTION
C       DATA. THE SAME GRID OF ENERGIES IS USED FOR ALL ELEMENTS.
C       EACH FILE IS ORGANIZED IN COLUMNS AND CONTAINS THE
C       FOLLOWING INFORMATION
C       1ST COLUMN: KINETIC ENERGIES IN eV (IN INCREASING ORDER).
C       2ND COLUMN: ELASTIC SCATTERING TOTAL CROSS SECTION FOR
C              ELECTRONS (barn).
C       3RD COLUMN: ELASTIC FIRST TRANSPORT CROSS SECTION FOR
C              ELECTRONS (barn).
C       4TH COLUMN: ELASTIC SECOND TRANSPORT CROSS SECTION FOR
C              ELECTRONS (barn).
C       5TH COLUMN: ELASTIC SCATTERING TOTAL CROSS SECTION FOR
C              POSITRONS (barn).
C       6TH COLUMN: ELASTIC FIRST TRANSPORT CROSS SECTION FOR
C              POSITRONS (barn).
C       7TH COLUMN: ELASTIC SECOND TRANSPORT CROSS SECTION FOR
C              POSITRONS (barn).
C       8TH COLUMN: RADIATIVE TOTAL STOPPING CROSS SECTION FOR
C              ELECTRONS (eV.barn).
C  *92 FILES NAMED 'PENPHZZ.TAB' WITH ZZ=ATOMIC NUMBER (01-92),
C       ORGANIZED IN COLUMNS. THE GRID OF ENERGIES FOR EACH
C       ELEMENT IS OBTAINED BY MERGING A GENERIC GRID (THE SAME
C       FOR ALL ELEMENTS) WITH THE GRID OF ABSORPTION EDGES OF
C       THE ELEMENT. MOREOVER, BETWEEN EACH PAIR OF CONSECUTIVE
C       ABSORPTION EDGES THERE IS AT LEAST ONE GRID POINT.
C       EACH FILE CONTAINS THE FOLLOWING QUANTITIES
C       1ST COLUMN: PHOTON ENERGIES IN eV (IN INCREASING ORDER).
C       2ND COLUMN: PHOTOELECTRIC TOTAL CROSS SECTION (barn).
C       3RD COLUMN: PAIR PRODUCTION CROSS SECTION (barn).
C  *'PENINCH.TAB': ONE-ELECTRON COMPTON PROFILES FOR P=0 OF THE
C       ELEMENTS ZZ=1 TO 92. EACH LINE CONTAINS DATA PERTAINING
C       TO AN ATOMIC ELECTRON SHELL.
C       1ST COLUMN: ATOMIC NUMBER OF THE ELEMENT.
C       2ND COLUMN: ELECTRON SHELL.
C       3RD COLUMN: NUMBER OF ELECTRONS IN THE SHELL.
C       4TH COLUMN: VALUE OF THE ONE-ELECTRON COMPTON PROFILE
C                   FOR P=0 (IN ATOMIC UNITS).
C       5TH COLUMN: IONIZATION ENERGY (IN eV).
C
C     TO OBTAIN THE EXECUTABLE FILE 'MATERIAL.EXE' COMPILE AND
C  LINK THE SOURCE FILES 'MATERIAL.FOR' AND 'PENELOPE.FOR'. THE
C  DATABASE FILES AND THE EXECUTABLE CODE 'MATERIAL.EXE' MUST BE
C  IN THE SAME DIRECTORY.
C
C  **************************************************************
C                       PROGRAM MATERIAL
C  **************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), CHARACTER*2 (L),
     1   INTEGER*4 (I-K,M-N)
      CHARACTER*1 LZ(10),L1,L2,L3
      CHARACTER*12 LFILE,NULL
      CHARACTER*80 PFILE,NAME1
      CHARACTER*60 NAME
      CHARACTER*42 LNAME
      PARAMETER (A0B=5.29177249D-9, HREV=27.2113961D0,
     1 AVOG=6.0221367D23, REV=5.1099906D5, SL=137.0359895D0)
      PARAMETER (PI=3.1415926535897932D0, FOURPI=4.0D0*PI)
      PARAMETER (NO=400, NOMAX=30, NTAB=180, NPH=400, NSHC=30)
      DIMENSION F(NO),UI(NO),FJ0(NO),G(NO),WI(NO),GJ0(NO),
     1    FJ0T(25),IZFL(NO),IZGL(NO),IBE(25),IZS(25)
      DIMENSION EE(NPH),CRAY(NPH),CCOM(NPH),CPHO(NPH),CPAI(NPH)
      DIMENSION      CS0E(NTAB),CS1E(NTAB),CS2E(NTAB),STCE(NTAB),
     1    STRE(NTAB),CS0P(NTAB),CS1P(NTAB),CS2P(NTAB),STCP(NTAB),
     2    STRP(NTAB)
      DIMENSION  CS0EA(NTAB),CS1EA(NTAB),CS2EA(NTAB),STREA(NTAB),
     1           CS0PA(NTAB),CS1PA(NTAB),CS2PA(NTAB)
      DIMENSION EGRID(16)
      COMMON/SHELLD/FC(NSHC),UIC(NSHC),FJ0C(NSHC),NSHELL
      COMMON/CCOMP/STF(15),IZM(15),NELEM
C  ****  ELEMENT DATA.
      COMMON/DELEM/LAS(99)
      COMMON/CATWG/ATW(99)
      COMMON/CMEXP/EPX(99)
C
      DATA LZ/'0','1','2','3','4','5','6','7','8','9'/
      DATA EGRID/1.0D0,1.25D0,1.50D0,1.75D0,2.00D0,2.50D0,3.00D0,
     1 3.50D0,4.00D0,4.50D0,5.00D0,6.00D0,7.00D0,8.00D0,9.00D0,
     2 1.00D1/
C
      WRITE(6,*) ' ENTER MATERIAL NAME, FOR YOUR INFORMATION ',
     1  '(NO MORE THAN 42 CHARACTERS) ...'
      READ(5,'(A42)') LNAME
      WRITE(6,'(1X,'' MATERIAL: '',A42)') LNAME
C
      WRITE(6,*) '      '
      WRITE(6,*) ' SELECT ONE OPTION (1 OR 2):'
      WRITE(6,*) '   1: ENTER COMPOSITION DATA FROM THE KEYBOARD'
      WRITE(6,*) '   2: READ THEM FROM THE FILE COMPDATA.TAB'
      READ(5,*) IREAD
      IF(IREAD.EQ.2) GO TO 900
C
      WRITE(6,*) '      '
      WRITE(6,*) ' CHEMICAL FORMULA:'
      WRITE(6,*) ' NUMBER OF ELEMENTS IN THE MOLECULE ...'
      READ(5,*) NELEM
      IF(NELEM.LT.1) STOP
      WRITE(6,'(1X,'' NUMBER OF ELEMENTS = '',I2)') NELEM
      IF(NELEM.GT.15) THEN
        WRITE(6,*) ' *** NELEM CANNOT BE LARGER THAN 15.'
        STOP
      ENDIF
C
      ZT=0.0D0
      AT=0.0D0
      EXPT=0.0D0
C
      STFT=0.0D0
      DO 1 I=1,NELEM
      IF(I.EQ.1) THEN
        WRITE(6,2001)
 2001 FORMAT(1X,' ENTER ATOMIC NUMBER AND NUMBER OF ATOMS/MOLEC',
     1  'ULE OF THE FIRST ELEMENT ...')
      ELSE IF(I.EQ.2) THEN
        WRITE(6,2002)
 2002 FORMAT(1X,' ENTER ATOMIC NUMBER AND NUMBER OF ATOMS/MOLEC',
     1  'ULE OF THE SECOND ELEMENT ...')
      ELSE IF(I.EQ.3) THEN
        WRITE(6,2003)
 2003 FORMAT(1X,' ENTER ATOMIC NUMBER AND NUMBER OF ATOMS/MOLEC',
     1  'ULE OF THE THIRD ELEMENT ...')
      ELSE
        WRITE(6,2004) I
 2004 FORMAT(1X,' ENTER ATOMIC NUMBER AND NUMBER OF ATOMS/MOLEC',
     1  'ULE OF THE',I3,'-TH ELEMENT ...')
      ENDIF
      READ(5,*) IZZ,STF(I)
      WRITE(6,'(1X,'' ELEMENT: '',A2,'' (Z='',I2,''), ATOMS/'',
     1  ''MOLECULE ='',1P,E12.5)') LAS(IZZ),IZZ,STF(I)
      IF(IZZ.LT.1.OR.IZZ.GT.92) STOP
      IF(STF(I).LE.0.0D0) STOP
      STFT=STFT+STF(I)
      ZT=ZT+IZZ*STF(I)
      AT=AT+ATW(IZZ)*STF(I)
      EXPT=EXPT+IZZ*DLOG(EPX(IZZ))*STF(I)
      IZM(I)=IZZ
    1 CONTINUE
C
      WRITE(6,*) '      '
      WRITE(6,*) ' ENTER MASS DENSITY (g/cm**3) ...'
      READ(5,*) RHO
      WRITE(6,'(1X,'' DENSITY = '',1P,E12.5,'' g/cm**3'')') RHO
      IF(RHO.LE.0.0D0) STOP
C
      EXPT=DEXP(EXPT/ZT)
      VMOL=AVOG*RHO/AT
      WRITE(6,*) '      '
      WRITE(6,'(1X,'' THE CALCULATED MEAN EXCITATION ENERGY I'',
     1  '' IS '',1P,E12.5,'' eV'')') EXPT
      WRITE(6,*) ' DO YOU WANT TO CHANGE IT?   (1=YES,2=NO)'
      READ(5,*) IYESNO
      IF(IYESNO.EQ.1) THEN
        WRITE(6,*) ' ENTER MEAN EXCITATION ENERGY (eV) ...'
        READ(5,*) EXPT
        WRITE(6,'(1X,'' MEAN EXCITATION ENERGY ='',1P,E12.5,
     1    '' eV'')') EXPT
        IF(EXPT.LT.1.0D0) STOP
      ENDIF
      GO TO 906
C
C  ************  MATERIAL DATA READ FROM FILE COMPDATA.TAB.
C
  900 CONTINUE
      WRITE(6,*) ' ENTER MATERIAL IDENTIFICATION NUMBER ...'
      READ(5,*) IDNUM
C
      OPEN(7,FILE='compdata.tab')
      DO 901 I=1,15
      READ(7,'(A80)') NAME1
  901 CONTINUE
      DO 903 K1=1,300
      READ(7,'(I3,1X,A60)',END=903) IORD,NAME
      READ(7,*) NELEM,ZOA,EXPT,RHO
      IF(NELEM.GT.15) THEN
        WRITE(6,*) ' *** NELEM CANNOT BE LARGER THAN 15.'
        STOP
      ENDIF
      IF(NELEM.LT.1) STOP
      DO 902 I=1,NELEM
      READ(7,*) IZM(I),XNOTH,STF(I)
  902 CONTINUE
      IF(IORD.EQ.IDNUM) GO TO 904
  903 CONTINUE
      STOP
  904 CONTINUE
      CLOSE(UNIT=7)
      WRITE(6,'(/2X,I3,1X,A60)') IORD,NAME
C
      ZT=0.0D0
      AT=0.0D0
      STFT=0.0D0
      DO 905 I=1,NELEM
      IZZ=IZM(I)
      WRITE(6,'(1X,'' ELEMENT: '',A2,'' (Z='',I2,''), ATOMS/'',
     1  ''MOLECULE ='',1P,E12.5)') LAS(IZZ),IZZ,STF(I)
      IF(IZZ.LT.1.OR.IZZ.GT.92) STOP
      IF(STF(I).LE.0.0D0) STOP
      STFT=STFT+STF(I)
      ZT=ZT+IZZ*STF(I)
      AT=AT+ATW(IZZ)*STF(I)
  905 CONTINUE
      WRITE(6,'(/1X,'' DENSITY = '',1P,E12.5,'' g/cm**3'')') RHO
      WRITE(6,'(/1X,'' NUMBER OF ELECTRONS PER MOLECULE = '',
     1  1P,E12.5)') ZT
      IF(RHO.LE.0.0D0) STOP
      WRITE(6,'(1X,'' MEAN EXCITATION ENERGY ='',1P,E12.5,
     1  '' eV'')') EXPT
      VMOL=AVOG*RHO/AT
C
C  ****  OPTICAL OSCILLATOR STRENGTH AND COMPTON PROFILE.
C
  906 CONTINUE
      NOS=0
      FT=0.0D0
      FCB=0.0D0
      DO 3 I=1,NELEM
      IZZ=IZM(I)
      CALL BINDEN(IZZ,IBE,IZS,NSH)
C  ****  CHECKING THE CONSISTENCY OF BINDING ENERGIES CODED IN
C        PENELOPE AND LISTED IN 'PENINCH.TAB'.
      OPEN(7,FILE='peninch.tab')
      DO 907 K=1,14
      READ(7,'(A12)') NULL
  907 CONTINUE
      NSHT=0
      ICOMP=11111
      DO 908 K=1,1322
      READ(7,'(1X,I2,1X,3A1,1X,I1,E9.2,I7)') IZZT,L1,L2,L3,IZST,
     1  CP,IBET
      IF(IZZT.EQ.IZZ) THEN
        NSHT=NSHT+1
        ICOMP=0
        IF(IZST.NE.IZS(NSHT)) ICOMP=1
        IF(IBET.NE.IBE(NSHT)) ICOMP=1
        IF(CP.LT.1.0D-16) NSHT=NSHT+10
        FJ0T(NSHT)=CP
      ENDIF
  908 CONTINUE
      CLOSE(UNIT=7)
      IF(NSHT.NE.NSH.OR.ICOMP.NE.0) THEN
        WRITE(6,'(/1X,''**** INCONSISTENT DATA IN FILE PENIN'',
     1  ''CH.TAB. JOB ABORTED.'')')
      ENDIF
C
      DO 2 K=1,NSH
      TST=IZS(K)*STF(I)
      IF(TST.LT.1.0D-6) GO TO 2
      NOS=NOS+1
      IF(NOS.GT.NO) THEN
        WRITE(6,*) ' TOO MANY OSCILLATORS. STOP.'
        STOP
      ENDIF
      UI(NOS)=IBE(K)
      F(NOS)=TST
      IF(IBE(K).LT.13) FCB=FCB+TST
      FJ0(NOS)=FJ0T(K)
      IF(K.EQ.1.AND.IZZ.GT.6) THEN
        IZFL(NOS)=IZZ
      ELSE
        IZFL(NOS)=0
      ENDIF
      FT=FT+F(NOS)
    2 CONTINUE
    3 CONTINUE
      FACTOR=FT/ZT
      DO 4 I=1,NOS
      F(I)=FACTOR*F(I)
    4 CONTINUE
C  ****  OSCILLATORS ARE SORTED BY INCREASING ENERGIES.
      RGROUP=1.0005D0
    5 CONTINUE
      FT=0.0D0
      IF(NOS.GT.1) THEN
        DO 7 I=1,NOS-1
        DO 6 J=I+1,NOS
        IF(UI(I).LT.UI(J)) GO TO 6
        SAVE=UI(I)
        UI(I)=UI(J)
        UI(J)=SAVE
        SAVE=F(I)
        F(I)=F(J)
        F(J)=SAVE
        SAVE=FJ0(I)
        FJ0(I)=FJ0(J)
        FJ0(J)=SAVE
        ISAVE=IZFL(I)
        IZFL(I)=IZFL(J)
        IZFL(J)=ISAVE
    6   CONTINUE
        FT=FT+F(I)
        IF(F(I).LT.1.0D-6) STOP
    7   CONTINUE
      ENDIF
      FT=FT+F(NOS)
      IF(F(NOS).LT.1.0D-6) STOP
      IF(DABS(FT-ZT).GT.1.0D-6) STOP
C  ****  RESONANCES DIFFERING BY A FACTOR LESS THAN RGROUP ARE
C        GROUPED IN A SINGLE OSCILLATOR.
      IELIM=0
      IF(NOS.GT.1) THEN
        DO 9 I=1,NOS-1
        IF(F(I).LT.0.0D0) GO TO 9
        IF(UI(I+1).GT.RGROUP*UI(I)) GO TO 9
        IF(IZFL(I+1).NE.IZFL(I)) GO TO 9
        UI(I)=DEXP((F(I)*DLOG(UI(I))+F(I+1)*DLOG(UI(I+1)))
     1       /(F(I)+F(I+1)))
        FJ0(I)=(F(I)*FJ0(I)+F(I+1)*FJ0(I+1))/(F(I)+F(I+1))
        F(I)=F(I)+F(I+1)
        DO 8 J=I+1,NOS-1
        F(J)=F(J+1)
        UI(J)=UI(J+1)
        FJ0(J)=FJ0(J+1)
        IZFL(J)=IZFL(J+1)
    8   CONTINUE
        IELIM=IELIM+1
        F(NOS)=-1.0D0
        UI(NOS)=1.0D16*IELIM
        FJ0(NOS)=0.0D0
        IZFL(NOS)=0
    9   CONTINUE
      ENDIF
      IF(IELIM.GT.0) THEN
        NOS=NOS-IELIM
        GO TO 5
      ENDIF
      IF(NOS.GT.NOMAX-1) THEN
        RGROUP=RGROUP**2
        GO TO 5
      ENDIF
C  ****  PLASMA ENERGY.
      OP2=FOURPI*ZT*VMOL*A0B**3
      OMEGA=HREV*DSQRT(OP2)
      OMEGA2=OMEGA*OMEGA
      WRITE(6,'(1X,'' PLASMA ENERGY = '',1P,E12.5,'' eV'')')
     1   OMEGA
      EPP=OMEGA*DSQRT(FCB/ZT)
      WRITE(6,'(/1X,'' ESTIMATED ENERGY AND OSCILLATOR STRENG'',
     1  ''TH OF THE PLASMON:''/2X,''WP ='',1P,E12.5,'' eV,  '',
     2  ''FP ='',E12.5)') EPP,FCB
C
      WRITE(6,*) '      '
      WRITE(6,2005)
 2005 FORMAT(1X,' ENTER THE ENERGY (IN eV) AND OSCILLATOR STREN',
     1  'GTH OF THE PLASMON ...'/1X,' (FOR INSULATORS, THESE QU',
     1  'ANTITIES SHOULD BE SET EQUAL TO ZERO)')
      READ(5,*) EP,FP
      WRITE(6,'(/1X,'' PLASMON ENERGY ='',1P,E12.5,'' eV, OSCI'',
     1  ''LLATOR STRENGTH ='',E12.5)') EP,FP
      IF(FP.GT.ZT) STOP
      IF(FP.GT.1.0D-12) THEN
        DO 10 I=1,NOS
        G(I)=F(I)
        WI(I)=UI(I)
        GJ0(I)=FJ0(I)
        IZGL(I)=IZFL(I)
   10   CONTINUE
        K=1
        F(1)=FP
        UI(1)=-EP
        FJ0(1)=0.75D0/SQRT(3.0D0*PI*PI*VMOL*A0B**3*FP)
        IZFL(1)=0
        I=0
   11   I=I+1
        IF(FP.GT.G(I)) THEN
          FP=FP-G(I)
        ELSE
          FK=G(I)-FP
          FP=0.0D0
          IF(FK.GT.1.0D-12) THEN
            K=K+1
            F(K)=FK
            UI(K)=WI(I)
            FJ0(K)=GJ0(I)
            IZFL(K)=IZGL(I)
          ENDIF
        ENDIF
        IF(I.LT.NOS) GO TO 11
        NOS=K
      ENDIF
C
C  ****  STERNHEIMER ADJUSTMENT FACTOR.
C
      IF(NOS.GT.1) THEN
        TST=ZT*DLOG(EXPT)
        AAL=0.5D0
        AAU=10.0D0
   12   AA=0.5D0*(AAL+AAU)
        SUM=0.0D0
        DO 13 I=1,NOS
        IF(UI(I).GT.0.0D0) THEN
          WI2=(AA*UI(I))**2+0.666666666666666D0*(F(I)/ZT)*OMEGA2
          WI(I)=DSQRT(WI2)
        ELSE
          WI(I)=DABS(UI(I))
        ENDIF
        SUM=SUM+F(I)*DLOG(WI(I))
   13   CONTINUE
        IF(SUM.LT.TST) THEN
          AAL=AA
        ELSE
          AAU=AA
        ENDIF
        IF(AAU-AAL.GT.1.0D-6*AA) GO TO 12
      ELSE
        UI(1)=DABS(UI(1))
        BA=ZT*DLOG(DABS(UI(1)))
        AA=DEXP((ZT*DLOG(EXPT)-BA)/ZT)
        WI(1)=AA*UI(1)
      ENDIF
C
C  ****  OPTICAL OSCILLATOR STRENGTH PARAMETERS.
C
      WRITE(6,*) '      '
      WRITE(6,'(1X,'' NUMBER OF OSCILLATORS = '',I3)') NOS
      TST=0.0D0
      DO 14 I=1,NOS
      UII=DMAX1(UI(I),0.0D0)
      WRITE(6,2006) I,F(I),I,UII,I,WI(I)
 2006 FORMAT(3X,'F(',I2,') =',1P,E12.5,',   U(',I2,') =',E12.5,
     1  ' eV,   W(',I2,') =',E12.5,' eV')
      WI(I)=WI(I)/HREV
      TST=TST+F(I)*DLOG(WI(I))
   14 CONTINUE
      WRITE(6,'(1X,'' GROUPING FACTOR = '',1P,E12.5)') RGROUP
      WRITE(6,'(1X,'' STERNHEIMER ADJUSTMENT FACTOR = '',1P,
     1   E12.5)') AA
      TEXPT=DEXP(TST/ZT)*HREV
      IF(DABS(TEXPT-EXPT).GT.1.0D-6*EXPT) THEN
        WRITE(6,'(/1X,'' MEAN EXCITATION ENERGY = '',1P,E12.5,
     1    '' eV'')') EXPT
        WRITE(6,2007) TEXPT
 2007   FORMAT(/1X,' *** INCONSISTENT OPTICAL OSCILLATOR STRENG',
     1  'TH PARAMETERS.'/5X,' CALCULATED MEAN EXCITATION ENERGY',
     2  ' =',1P,E12.5,' eV')
        STOP
      ENDIF
C
C  ****  COMPTON CROSS SECTION PARAMETERS.
C
      NSHELL=NOS
      DO 15 I=1,NOS
      FC(I)=F(I)
      UIC(I)=UI(I)
      FJ0C(I)=FJ0(I)*SL
   15 CONTINUE
C
C  ************  PENELOPE'S INPUT FILE.
C
      WRITE(6,*) '      '
      WRITE(6,*) ' PENELOPE''S MATERIAL DATA FILE IS BEING',
     1  ' CREATED'
      WRITE(6,*) ' ENTER PATH+NAME FOR THIS FILE (UP TO 80 CHAR',
     1  'ACTERS) ...'
      READ(5,'(A80)') PFILE
      OPEN(7,FILE=PFILE)
C
      WRITE(7,1001) LNAME
 1001 FORMAT('P1   MATERIAL:(',A42,'-DBASE)')
      WRITE(7,1002) RHO
 1002 FORMAT('P2   MASS DENSITY=(',1P,E11.4,') g/cm**3')
      WRITE(7,1003) NELEM
 1003 FORMAT('P3   NUMBER OF ELEMENTS IN THE MOLECULE=(',
     1  I2,') /.LE.15/')
      DO 101 I=1,NELEM
      WRITE(7,1004) IZM(I),STF(I)
 1004 FORMAT('P4+  ATOMIC NUMBER=(',I2,'),ATOMS/MOLECULE=(',
     1  1P,E13.6,')')
  101 CONTINUE
      WRITE(7,1005) EXPT
 1005 FORMAT('P5   MEAN EXCITATION ENERGY=(',F8.2,') eV')
      WRITE(7,1006) NOS
 1006 FORMAT('P6   NUMBER OF OSCILLATORS=(',I2,') /.LE.30/')
      DO 102 I=1,NOS
      WRITE(7,1007) F(I),UI(I),FJ0(I),IZFL(I)
 1007 FORMAT('P7+  (',1P,E13.6,',',E13.6,',',E10.3,',',I2,') O',
     1  'SCILLATOR DATA')
  102 CONTINUE
C  ****  ENERGY GRID.
      NDATE=0
      IGRID=0
      FGRID=100.0D0
  103 IGRID=IGRID+1
      EV=EGRID(IGRID)*FGRID
      IF(IGRID.EQ.16) THEN
        IGRID=1
        FGRID=10.0D0*FGRID
      ENDIF
      IF(EV.LT.99.0D0) GO TO 103
      NDATE=NDATE+1
      EE(NDATE)=EV
      IF(EV.LT.1.0D9) GO TO 103
C
C  ************  READING OF DATABASE FILES.
C
C  ****  ELECTRON AND POSITRON CROSS SECTION.
C
      DO 201 I=1,NTAB
      CS0E(I)=0.0D0
      CS1E(I)=0.0D0
      CS2E(I)=0.0D0
      STCE(I)=0.0D0
      STRE(I)=0.0D0
      CS0P(I)=0.0D0
      CS1P(I)=0.0D0
      CS2P(I)=0.0D0
      STCP(I)=0.0D0
      STRP(I)=0.0D0
  201 CONTINUE
C
      DO 205 IELEM=1,NELEM
      IZ=IZM(IELEM)
      IZ2=IZ-10*(IZ/10)
      IZ1=(IZ-IZ2)/10
      L1=LZ(IZ1+1)
      L2=LZ(IZ2+1)
      LFILE='penep'//L1//L2//'.tab'
      OPEN(3,FILE=LFILE)
      READ(3,*) IIZ
      IF(IIZ.NE.IZ) THEN
        WRITE(7,'(I3)') IIZ
        WRITE(6,'(1X,'' ERROR IN FILE '',A12)') LFILE
        STOP
      ENDIF
      DO 202 I=1,NDATE
      READ(3,*,END=203) EINP,CS0EA(I),CS1EA(I),CS2EA(I),CS0PA(I),
     1  CS1PA(I),CS2PA(I),STREA(I)
      IF(DABS(EINP-EE(I)).GT.1.0D-1) THEN
        WRITE(7,'(1P,E9.2,7E10.3)') EINP,CS0EA(I),CS1EA(I),
     1    CS2EA(I),CS0PA(I),CS1PA(I),CS2PA(I),STREA(I)
        STOP
      ENDIF
  202 CONTINUE
  203 CONTINUE
      CLOSE(UNIT=3)
C
      DO 204 I=1,NDATE
      CS0E(I)=CS0E(I)+STF(IELEM)*CS0EA(I)*1.0D-24
      CS1E(I)=CS1E(I)+STF(IELEM)*CS1EA(I)*1.0D-24
      CS2E(I)=CS2E(I)+STF(IELEM)*CS2EA(I)*1.0D-24
      CS0P(I)=CS0P(I)+STF(IELEM)*CS0PA(I)*1.0D-24
      CS1P(I)=CS1P(I)+STF(IELEM)*CS1PA(I)*1.0D-24
      CS2P(I)=CS2P(I)+STF(IELEM)*CS2PA(I)*1.0D-24
      STRE(I)=STRE(I)+STF(IELEM)*STREA(I)*1.0D-24
      T=DLOG(1.0D0+1.0D6*EE(I)/(IZ*IZ*REV))
      FPOS=1.0D0-DEXP(-T*(1.2359D-1-T*(6.1274D-2-T*(3.1516D-2-T
     1   *(7.7446D-3-T*(1.0595D-3-T*(7.0568D-5-T*1.8080D-6)))))))
      STRP(I)=STRP(I)+FPOS*STF(IELEM)*STREA(I)*1.0D-24
  204 CONTINUE
  205 CONTINUE
C
      DO 206 I=1,NDATE
      EAU=EE(I)/HREV
      CALL PEINEL(ZT,F,WI,OP2,0.0D0,EAU,CS0,CS1,CS2,NOS,-1)
      STCE(I)=STCE(I)+CS1*HREV*A0B**2
      CALL PEINEL(ZT,F,WI,OP2,0.0D0,EAU,CS0,CS1,CS2,NOS,+1)
      STCP(I)=STCP(I)+CS1*HREV*A0B**2
  206 CONTINUE
C
      WRITE(7,1008) NDATE
 1008 FORMAT('P8   NUMBER OF INPUT ENERGIES=(',I3,') FOR',
     1  ' ELECTRONS /.LT.180/')
      DO 207 I=1,NDATE
      CS0E(I)=1.0D6*RHO/(VMOL*CS0E(I))
      CS1E(I)=1.0D6*RHO/(VMOL*CS1E(I))
      CS2E(I)=1.0D6*RHO/(VMOL*CS2E(I))
      STCE(I)=1.0D-6*VMOL*STCE(I)/RHO
      STRE(I)=1.0D-6*VMOL*STRE(I)/RHO
      WRITE(7,1009) EE(I),CS0E(I),CS1E(I),CS2E(I),STCE(I),
     1  STRE(I)
 1009 FORMAT(1P,E9.2,5E11.3)
  207 CONTINUE
C
      WRITE(7,1010) NDATE
 1010 FORMAT('P10  NUMBER OF INPUT ENERGIES=(',I3,') FOR',
     1  ' POSITRONS /.LT.180/')
      DO 208 I=1,NDATE
      CS0P(I)=1.0D6*RHO/(VMOL*CS0P(I))
      CS1P(I)=1.0D6*RHO/(VMOL*CS1P(I))
      CS2P(I)=1.0D6*RHO/(VMOL*CS2P(I))
      STCP(I)=1.0D-6*VMOL*STCP(I)/RHO
      STRP(I)=1.0D-6*VMOL*STRP(I)/RHO
      WRITE(7,1009) EE(I),CS0P(I),CS1P(I),CS2P(I),STCP(I),
     1  STRP(I)
  208 CONTINUE
C
C  ****  PHOTON ATTENUATION COEFFICIENTS.
C
      CALL PHOTCS(EE,CRAY,CCOM,CPHO,CPAI,NDATP)
      WRITE(7,1011) NDATP
 1011 FORMAT('P12  NUMBER OF INPUT ENERGIES=(',I3,') FOR',
     1  ' PHOTONS /.LT.400/')
      DO 209 I=1,NDATP
      ATCRAY=VMOL*CRAY(I)*1.0D-24/RHO
      ATCCOM=VMOL*CCOM(I)*1.0D-24/RHO
      ATCPHO=VMOL*CPHO(I)*1.0D-24/RHO
      IF(CPAI(I).GT.2.0D-35) THEN
        ATCPAI=VMOL*CPAI(I)*1.0D-24/RHO
      ELSE
        ATCPAI=1.0D-35
      ENDIF
      WRITE(7,1012) EE(I),ATCRAY,ATCCOM,ATCPHO,ATCPAI
 1012 FORMAT(1P,E10.3,5E11.3)
  209 CONTINUE
C
      CLOSE(UNIT=7)
      STOP
      END
C  **************************************************************
C                        SUBROUTINE PHOTCS
C  **************************************************************
      SUBROUTINE PHOTCS(E,CRAY,CCOM,CPHO,CPAI,NDAT)
C
C     THIS SUBROUTINE GIVES TOTAL PHOTON INTERACTION CROSS SEC-
C  TIONS FOR ELEMENTS AND COMPOUNDS, FOR PHOTON ENERGIES LARGER
C  THAN 1 keV. MOLECULAR CROSS SECTIONS ARE CALCULATED FROM THE
C  ATOMIC CROSS SECTIONS OF THE ELEMENTS BY MEANS OF THE ADDITIV-
C  ITY RULE.
C
C     THE COHERENT AND INCOHERENT CROSS SECTIONS ARE OBTAINED
C  FROM THE ANALYTICAL ATOMIC FORM FACTORS AND INCOHERENT SCAT-
C  TERING FUNCTIONS DESCRIBED BY J. BARO ET AL., RADIAT. PHYS.
C  CHEM. 44 (1994) 531-552. THE CROSS SECTIONS FOR PHOTOELECTRIC
C  ABSORPTION AND ELECTRON-POSITRON PAIR PRODUCTION ARE READ FROM
C  THE DATABASE FILES 'PENPHZZ.TAB'.
C
C  INPUT (THROUGH COMMON/CCOMP/):
C     NELEM .... NUMBER OF DIFERENT ELEMENTS IN THE MOLECULE.
C     IZM(I) ... ATOMIC NUMBER OF THE I-TH ELEMENT.
C     STF(I) ... NUMBER OF ATOMS OF THE I-TH ELEMENT IN A MOLE-
C                CULE, I.E. THE STOICHIOMETRIC INDEX.
C  OTHER INPUT PARAMETERS (COMMON /SHELLD/):
C     FZ(I) .... SHELL OCCUPATION NUMBERS,
C     UI(I) .... SHELL IONIZATION ENERGIES (IN eV),
C     FJ0(I) ... MAXIMUM VALUE OF THE ONE-ELECTRON COMPTON PRO-
C                FILES (IN UNITS OF MC).
C     NSHELL ... NUMBER OF SHELLS.
C
C
C  OUTPUT (ARGUMENTS):
C     NDAT ...... NUMBER OF ENERGY GRID POINTS.
C     E(I) ...... ENERGY (IN eV).
C     CRAY(I) ... COHERENT SCATTERING CROSS SECTION (IN barn).
C     CCOM(I) ... INCOHERENT SCATTERING CROSS SECTION (IN barn).
C     CPHO(I) ... PHOTOABSORPTION CROSS SECTION (IN barn).
C     CPAI(I) ... PAIR-PRODUCTION CROSS SECTION (IN barn).
C
C     THE ARGUMENTS E, CRAY, CCOM, CPHO AND CPAI ARE ARRAYS OF
C  DIMENSION NPH=400.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      CHARACTER*1 LZ(10),L1,L2
      CHARACTER*12 LPFILE
      PARAMETER(EPS=1.0D-10,NPH=400,NSHC=30)
      DIMENSION E(NPH),CRAY(NPH),CCOM(NPH),CPHO(NPH),CPAI(NPH)
      DIMENSION Y1(NPH),Z1(NPH),E2(NPH),Y2(NPH),Z2(NPH),ET(NPH),
     1          YT(NPH),ZT(NPH)
C
      COMMON/CCOMP/STF(15),IZM(15),NELEM
      COMMON/SHELLD/FZ(NSHC),UI(NSHC),FJ0(NSHC),NSHELL
      DATA LZ/'0','1','2','3','4','5','6','7','8','9'/
C
C  ****  READ THE DATABASE FILES.
C
      IZMAX=0
      DO 5 IELEM=1,NELEM
      IZ=IZM(IELEM)
      IF(IZ.GT.IZMAX) IZMAX=IZ
      IZ2=IZ-10*(IZ/10)
      IZ1=(IZ-IZ2)/10
      L1=LZ(IZ1+1)
      L2=LZ(IZ2+1)
      LPFILE='penph'//L1//L2//'.tab'
      OPEN(3,FILE=LPFILE)
      READ(3,*) IIZ
      IF(IIZ.NE.IZ) THEN
        WRITE(7,'(I3)') IIZ
        WRITE(6,'('' ERROR IN FILE '',A11)') LPFILE
        STOP
      ENDIF
      DO 1 I=1,NPH
      IF(I.EQ.1) THEN
        READ(3,*,END=2) E2(I),Y2(I)
        Z2(I)=1.0D-35
      ELSE IF(E2(I-1).LT.9.99D5) THEN
        READ(3,*,END=2) E2(I),Y2(I)
        Z2(I)=1.0D-35
      ELSE
        READ(3,*,END=2) E2(I),Y2(I),Z2(I)
      ENDIF
      IF(E2(I).GT.1.01D9) GO TO 2
      N2=I
      Y2(I)=STF(IELEM)*Y2(I)
      Z2(I)=STF(IELEM)*Z2(I)
    1 CONTINUE
    2 CONTINUE
      CLOSE(UNIT=3)
C
      IF(IELEM.EQ.1) THEN
        N1=N2
        DO 3 I=1,N1
        E(I)=E2(I)
        Y1(I)=Y2(I)
        Z1(I)=Z2(I)
    3   CONTINUE
      ELSE
        CALL MERGE2(E,Y1,Z1,E2,Y2,Z2,ET,YT,ZT,N1,N2,N)
        N1=N
        DO 4 I=1,N1
        E(I)=ET(I)
        Y1(I)=YT(I)
        Z1(I)=ZT(I)
    4   CONTINUE
      ENDIF
    5 CONTINUE
C
C  ****  HIGH-ENERGY LIMIT OF THE RAYLEIGH CROSS SECTION.
C
      ECUT=DMIN1(5.0D5*IZMAX,1.0D7)
      RACS=0.0D0
      DO 6 J=1,NELEM
      ZZ=IZM(J)
      IZ=ZZ+0.5D0
      CALL PERAYL(IZ,ECUT,CS1)
      RACS=RACS+CS1*STF(J)
    6 CONTINUE
C
C  ****  COMPTON AND RAYLEIGH TOTAL CROSS SECTIONS.
C
      NDAT=N1
      DO 8 I=1,NDAT
      EE=E(I)
      IF(EE.LT.ECUT) THEN
        IRAY=1
      ELSE
        IRAY=0
      ENDIF
      CRAY(I)=0.0D0
      CCOM(I)=0.0D0
      CPHO(I)=Y1(I)
      CPAI(I)=Z1(I)
      DO 7 J=1,NELEM
      ZZ=IZM(J)
      IZ=ZZ+0.5D0
      IF(IRAY.EQ.1) THEN
        CALL PERAYL(IZ,EE,CS1)
        CRAY(I)=CRAY(I)+CS1*STF(J)
      ENDIF
    7 CONTINUE
      IF(IRAY.EQ.0) THEN
        CRAY(I)=RACS*(ECUT/EE)**2
      ENDIF
      CALL PSCOMP(EE,CS2)
      CCOM(I)=CS2
    8 CONTINUE
C
      RETURN
      END
C  **************************************************************
C                        SUBROUTINE MERGE2
C  **************************************************************
      SUBROUTINE MERGE2(X1,Y1,Z1,X2,Y2,Z2,X,Y,Z,N1,N2,N)
C
C     THIS SUBROUTINE MERGES TWO TABLES (X1,Y1,Z1), (X2,Y2,Z2) OF
C  TWO FUNCTIONS TO PRODUCE A TABLE (X,Y,Z) OF THE SUM OF THESE
C  FUNCTIONS, WITH ABSCISSAS IN INCREASING ORDER. N1, N2 AND N
C  ARE THE NUMBERS OF GRID POINTS IN THE INPUT AND MERGED TABLES.
C  A DISCONTINUITY IN THE FUNCTION IS DESCRIBED BY GIVING TWICE
C  THE ABSCISSA. LOG-LOG LINEAR INTERPOLATION IS USED TO INTERPO-
C  LATE THE INPUT TABLES.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      PARAMETER(EPS=1.0D-10,NP=400)
      DIMENSION X1(NP),Y1(NP),Z1(NP),X2(NP),Y2(NP),Z2(NP),
     1          X(NP),Y(NP),Z(NP)
C
      CALL SORT2(X1,Y1,Z1,N1)
      CALL SORT2(X2,Y2,Z2,N2)
C
      DO 1 I1=1,N1
      X(I1)=X1(I1)
    1 CONTINUE
      N=N1
      DO 2 I2=1,N2
      XC=X2(I2)
      CALL FINDI(X1,XC,N1,I1)
      TST1=DABS(XC-X1(I1))
      TST2=DABS(XC-X1(I1+1))
      TST12=DMIN1(TST1,TST2)
      IF(I2.GT.1) THEN
        TST3=DABS(XC-X2(I2-1))
      ELSE
        TST3=1.0D0
      ENDIF
      IF(I2.LT.N2) THEN
        TST4=DABS(XC-X2(I2+1))
      ELSE
        TST4=1.0D0
      ENDIF
      TST34=DMIN1(TST3,TST4)
      TST=EPS*XC
      IF(TST34.GT.TST) THEN
        IF(TST12.GT.TST) THEN
          N=N+1
          X(N)=XC
        ENDIF
      ELSE
        N=N+1
        X(N)=XC
      ENDIF
    2 CONTINUE
C
C  ****  SORT AND CLEAN THE MERGED GRID.
C
    3 CONTINUE
      DO 5 I=1,N-1
      IMIN=I
      XMIN=X(I)
      DO 4 J=I+1,N
      IF(X(J).LT.XMIN) THEN
        IMIN=J
        XMIN=X(J)
      ENDIF
    4 CONTINUE
      SAVE=X(I)
      X(I)=X(IMIN)
      X(IMIN)=SAVE
    5 CONTINUE
C
      DO 6 I=1,N-2
      IF(X(I).GT.X(I+2)*(1.0D0-EPS)) THEN
        X(I+1)=X(N)
        N=N-1
        GO TO 3
      ENDIF
    6 CONTINUE
C
      DO 7 I=1,N
      XC=X(I)
      IF(I.LT.N) THEN
        IF(X(I).GT.X(I+1)*(1.0D0-EPS)) XC=X(I)*(1.0D0-EPS)
      ENDIF
      IF(I.GT.1) THEN
        IF(X(I).LT.X(I-1)*(1.0D0+EPS)) XC=X(I)*(1.0D0+EPS)
      ENDIF
      CALL FINDI(X1,XC,N1,J)
      IF(X1(J+1).GT.X1(J)+EPS) THEN
        YI1=DEXP(DLOG(Y1(J))+DLOG(XC/X1(J))*DLOG(Y1(J+1)/Y1(J))
     1   /DLOG(X1(J+1)/X1(J)))
        ZI1=DEXP(DLOG(Z1(J))+DLOG(XC/X1(J))*DLOG(Z1(J+1)/Z1(J))
     1   /DLOG(X1(J+1)/X1(J)))
      ELSE
        YI1=Y1(J)
        ZI1=Z1(J)
      ENDIF
      CALL FINDI(X2,XC,N2,J)
      IF(X2(J+1).GT.X2(J)+EPS) THEN
        YI2=DEXP(DLOG(Y2(J))+DLOG(XC/X2(J))*DLOG(Y2(J+1)/Y2(J))
     1   /DLOG(X2(J+1)/X2(J)))
        ZI2=DEXP(DLOG(Z2(J))+DLOG(XC/X2(J))*DLOG(Z2(J+1)/Z2(J))
     1   /DLOG(X2(J+1)/X2(J)))
      ELSE
        YI2=Y2(J)
        ZI2=Z2(J)
      ENDIF
      Y(I)=YI1+YI2
      Z(I)=ZI1+ZI2
      IF(Y(I).LT.1.0D-34) Y(I)=1.0D-35
      IF(Z(I).LT.1.0D-34) Z(I)=1.0D-35
    7 CONTINUE
C
      RETURN
      END
C  **************************************************************
C                        SUBROUTINE SORT2
C  **************************************************************
      SUBROUTINE SORT2(X,Y,Z,N)
C
C     THIS SUBROUTINE SORTS A TABLE (X,Y,Z), WITH N DATA POINTS,
C  OF TWO FUNCTIONS. A DISCONTINUITY IN ONE OF THE FUNCTIONS IS
C  DESCRIBED BY GIVING TWICE THE ABSCISSA.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      PARAMETER(EPS=1.0D-10,NP=400)
      DIMENSION X(NP),Y(NP),Z(NP),IORDER(NP)
C
      IF(N.EQ.1) RETURN
      DO 1 I=1,N
      IORDER(I)=I
      IF(Y(I).LT.1.0D-35) Y(I)=1.0D-35
      IF(Z(I).LT.1.0D-35) Z(I)=1.0D-35
    1 CONTINUE
C
      DO 3 I=1,N-1
      IMIN=I
      XMIN=X(I)
      DO 2 J=I+1,N
      IF(X(J).LT.XMIN) THEN
        IMIN=J
        XMIN=X(J)
      ENDIF
    2 CONTINUE
      SAVE=X(I)
      X(I)=X(IMIN)
      X(IMIN)=SAVE
      SAVE=Y(I)
      Y(I)=Y(IMIN)
      Y(IMIN)=SAVE
      SAVE=Z(I)
      Z(I)=Z(IMIN)
      Z(IMIN)=SAVE
      ISAVE=IORDER(I)
      IORDER(I)=IORDER(IMIN)
      IORDER(IMIN)=ISAVE
      IF(I.EQ.1) GO TO 3
      IF(IORDER(I).LT.IORDER(I-1)) THEN
        SAVE=X(I-1)
        X(I-1)=X(I)
        X(I)=SAVE
        SAVE=Y(I-1)
        Y(I-1)=Y(I)
        Y(I)=SAVE
        SAVE=Z(I-1)
        Z(I-1)=Z(I)
        Z(I)=SAVE
        ISAVE=IORDER(I-1)
        IORDER(I-1)=IORDER(I)
        IORDER(I)=ISAVE
      ENDIF
    3 CONTINUE
      I=N
      IF(IORDER(I).LT.IORDER(I-1)) THEN
        SAVE=X(I-1)
        X(I-1)=X(I)
        X(I)=SAVE
        SAVE=Y(I-1)
        Y(I-1)=Y(I)
        Y(I)=SAVE
        SAVE=Z(I-1)
        Z(I-1)=Z(I)
        Z(I)=SAVE
      ENDIF
      RETURN
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C             PHOTON ANALYTICAL CROSS SECTIONS                  C
C    OTHER SUBROUTINES IN THE PENELOPE.FOR FILE ARE REQUIRED    C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  **************************************************************
C                       SUBROUTINE BINDEN
C  **************************************************************
      SUBROUTINE BINDEN(IZ,IBE,IZS,NSH)
C
C     THIS SUBROUTINE GIVES BOUND SHELL IONIZATION ENERGIES IN EV
C  (IBE) AND OCCUPATION NUMBERS (IZS) FOR THE ELEMENT WITH ATOMIC
C  NUMBER IZ (.LE.92). NSH IS THE NUMBER OF OCCUPIED SHELLS.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      DIMENSION IBE(25),IZS(25)
      COMMON/CBIND/IB(25,92),NOC(25)
C
      IZT=IZ
      DO 1 I=1,25
      IBE(I)=IB(I,IZ)
      IZS(I)=NOC(I)
      IZT=IZT-IZS(I)
      IF(IZT.GT.0) GO TO 1
      IZS(I)=IZS(I)+IZT
      NSH=I
      RETURN
    1 CONTINUE
      RETURN
      END
C  **************************************************************
C                       SUBROUTINE PERAYL
C  **************************************************************
      SUBROUTINE PERAYL(IZ,E,CS)
C
C     ATOMIC CROSS SECTION FOR COHERENT (RAYLEIGH) SCATTERING
C  OBTAINED FROM THE ANALYTICAL FORM FACTOR.
C
C  INPUT ARGUMENTS:
C     IZ ....... ATOMIC NUMBER,
C     E ........ PHOTON ENERGY (EV).
C
C  OUTPUT ARGUMENT:
C     CS ....... COHERENT CROSS SECTION (BARN/ATOM).
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      PARAMETER (PI=3.1415926535897932D0, REV=5.1099906D5,
     1 ELRAD=2.81794092D-13, PIELR2=PI*ELRAD*ELRAD*1.0D24)
      COMMON/CFORM/A1(95),A2(95),A3(95),A4(95),A5(95)
      COMMON/COHER/A1F,A2F,A3F,A4F,A5F,TEKX2,IZZ
      EXTERNAL RAYDCS
C
      EC=DMIN1(E,5.0D5*IZ)
      TEKX2=849.3315D0*(EC/REV)**2
      IZZ=IZ
C  ****  PARAMETERS OF THE ANALYTICAL FORM FACTOR.
      IF(IZ.GT.95) STOP 10
      A1F=A1(IZ)
      A2F=A2(IZ)
      A3F=A3(IZ)
      A4F=A4(IZ)
      A5F=A5(IZ)
C  ****  RAYLEIGH CROSS SECTION.
      CALL GABQ(RAYDCS,-1.0D0,0.90D0,CS1,1.0D-5,IER)
      CALL GABQ(RAYDCS,0.90D0,0.99D0,CS2,1.0D-5,IER)
      CALL GABQ(RAYDCS,0.99D0,1.00D0,CS3,1.0D-5,IER)
      CS=PIELR2*IZ*IZ*(CS1+CS2+CS3)*(EC/E)**2
      RETURN
      END
C  **************************************************************
      FUNCTION RAYDCS(T)
C
C     RAYLEIGH DIFFERENTIAL CROSS SECTION. T=COS(THETA).
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      PARAMETER (SL=137.0359895D0)
      COMMON/COHER/A1F,A2F,A3F,A4F,A5F,TEKX2,IZ
      X2=TEKX2*(1.0D0-T)
      X=DSQRT(X2)
C  ****  FORM FACTOR DIVIDED BY Z.
      FA=(1.0D0+X2*(A1F+X*(A2F+X*A3F)))
     1  /(1.0D0+X2*(A4F+X2*A5F))**2
      IF(IZ.GT.10.AND.FA*IZ.LT.2.0D0) THEN
        PA=(IZ-0.3125D0)/SL
        PG=DSQRT(1.0D0-PA*PA)
        PQ=2.426311D-2*X/PA
        FB=DSIN(2.0D0*PG*DATAN(PQ))/(IZ*PG*PQ*(1.0D0+PQ*PQ)**PG)
        FA=DMAX1(FA,FB)
      ENDIF
C  ****  RAYLEIGH DIFFERENTIAL CROSS SECTION.
      RAYDCS=(1.0D0+T*T)*FA*FA
      RETURN
      END
C  **************************************************************
C                       SUBROUTINE PSCOMP
C  **************************************************************
      SUBROUTINE PSCOMP(E,CS)
C
C     CROSS SECTION FOR INCOHERENT (COMPTON) SCATTERING. IMPULSE
C  APPROXIMATION WITH ANALYTICAL COMPTON PROFILES.
C
C  INPUT ARGUMENTS:
C     E ........ PHOTON ENERGY (EV).
C  OTHER INPUT PARAMETERS (COMMON /SHELLD/):
C     FZ(I) .... SHELL OCCUPATION NUMBERS,
C     UI(I) .... SHELL IONIZATION ENERGIES (IN eV),
C     FJ0(I) ... MAXIMUM VALUES OF THE ONE-ELECTRON COMPTON PRO-
C                FILES (IN UNITS OF MC).
C     NSHELL ... NUMBER OF SHELLS.
C
C  OUTPUT ARGUMENT:
C     CS ....... INCOHERENT TOTAL CROSS SECTION (BARN/MOLECULE).
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      PARAMETER (NSHC=30)
      PARAMETER (PI=3.1415926535897932D0, REV=5.1099906D5,
     1 ELRAD=2.81794092D-13, PIELR2=PI*ELRAD*ELRAD*1.0D24)
      COMMON/SHELLD/FZ(NSHC),UI(NSHC),FJ0(NSHC),NSHELL
      COMMON/SAVEE/EE
      EXTERNAL COMDCS
C
      IF(E.LT.5.0D6) THEN
        EE=E
        CALL GABQ(COMDCS,-1.0D0,1.0D0,CS,1.0D-4,IER)
        CS=PIELR2*CS/E**2
      ELSE
C  ****  KLEIN-NISHINA TOTAL CROSS SECTION.
        EK=E/REV
        EK3=EK*EK
        EK2=1.0D0+EK+EK
        EK1=EK3-EK2-1.0D0
        T0=1.0D0/(1.0D0+EK+EK)
        CSL=0.5D0*EK3*T0*T0+EK2*T0+EK1*DLOG(T0)-1.0D0/T0
        CS=0.0D0
        DO 1 I=1,NSHELL
        TAU=(E-UI(I))/E
        IF(TAU.LT.T0) GO TO 1
        CSU=0.5D0*EK3*TAU*TAU+EK2*TAU+EK1*DLOG(TAU)-1.0D0/TAU
        CS=CS+FZ(I)*(CSU-CSL)
    1   CONTINUE
        CS=PIELR2*CS/(EK*EK3)
      ENDIF
      RETURN
      END
C  **************************************************************
C                        FUNCTION COMDCS
C  **************************************************************
      FUNCTION COMDCS(CDT)
C
C     DIFFERENTIAL CROSS SECTION FOR PHOTON COMPTON SCATTERING,
C  DIFFERENTIAL IN THE DIRECTION OF THE SCATTERED PHOTON.
C  INCOHERENT SCATTERING FUNCTION DERIVED FROM THE IMPULSE
C  APPROXIMATION USING ANALYTICAL COMPTON PROFILE.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      PARAMETER (NSHC=30)
      PARAMETER (REV=5.1099906D5)
      PARAMETER (D1=0.707107D0, D2=1.414214D0, D12=D1*D1)
      COMMON/SHELLD/FZ(NSHC),UI(NSHC),FJ0(NSHC),NSHELL
      COMMON/SAVEE/E
C
      CDT1=1.0D0-CDT
C  ****  ENERGY OF THE COMPTON LINE.
      EC=E/(1.0D0+(E/REV)*CDT1)
C  ****  INCOHERENT SCATTERING FUNCTION (ANALYTICAL PROFILE).
      SIA=0.0D0
      DO 1 I=1,NSHELL
      IF(E.LT.UI(I)) GO TO 1
      AUX=E*(E-UI(I))*CDT1
      PZIMAX=(AUX-REV*UI(I))/(REV*DSQRT(AUX+AUX+UI(I)**2))
      X=FJ0(I)*PZIMAX
      IF(X.GT.0.0D0) THEN
        SIAP=1.0D0-0.5D0*DEXP(D12-(D1+D2*X)**2)
      ELSE
        SIAP=0.5D0*DEXP(D12-(D1-D2*X)**2)
      ENDIF
      SIA=SIA+FZ(I)*SIAP
    1 CONTINUE
C  ****  KLEIN-NISHINA X-FACTOR.
      XKN=(E/EC)+(EC/E)-1.0D0+CDT*CDT
C  ****  DIFFERENTIAL CROSS SECTION.
      COMDCS=EC**2*XKN*SIA
      RETURN
      END
