C  **************************************************************
C                       FUNCTION PSTPBR
C
C  1999-09-08
C
C  Modified from PENELOPE PECALC and PBREM to fit into predpm to 
C  get a more accurate brems restricted radiative stopping power.
C  
C  This is somewhat cumbersome, because penelope gets both the
C  cross section and the radiative stopping power in the same
C  call to PEBREM. So we call that function in each of two loops:
C  one when we're getting the cross section, and one in getting
C  the stopping power.
C
C  **************************************************************
      real*8 function pstpbr(energy)
C
      implicit none
      real*8 energy
C
      real*8 zasum
      common /mater2/ zasum
      real*8 atno,atno2,atno21,mass,dens,excite
      common /mater/ atno,atno2,atno21,mass,dens,excite
      real*8 facion,facbre,facexc,faciom
      common /factor/ facion,facbre,facexc,faciom
      real*8 wcion,wcbre
      common /cutoff/ wcion,wcbre
C
      real*8 mc2,imc2,pi,hrev,a0b,sl,sl2,avog
      parameter (mc2=510.9991d3,imc2=1.d0/mc2)
C
      PARAMETER (PI=3.1415926535897932D0)
      PARAMETER (A0B=5.29177249D-9, HREV=27.2113961D0,
     + SL=137.0359895D0, SL2=SL*SL, AVOG=6.0221367D23)
C
      real*8 fcmtue,fccmie,zzeq,thbh,e,stp
      real*8 FF2,W1I,S0D,S1D,S2D,S1T,S2T
C
      real*8 rstpow
C
      FCCMIE=AVOG*dens*A0B**2/(zasum/atno)
      FCMTUE=1.0D-6*AVOG*A0B**2/(zasum/atno)
      ZZEQ=zasum/mass
C
C  cut-off for hard collisions in terms of hmc^2
C
      THBH=wcbre/HREV
C
C  get the radiative stopping power at this energy...
C
      STP=rstpow(energy)*1.d-6
      E=energy/hrev
C
C  get the cross section and the stopping power
C
      IF(wcbre.GE.10.0D0) THEN
        CALL PEBREM(ZZEQ,THBH,E,S0D,S1D,S2D,S1T,S2T)
        STP=STP/(HREV*FCMTUE)
        FF2=STP/S1T
        W1I=FCCMIE*HREV*FF2*S1D
      ELSE
        W1I=0.0D0
      ENDIF
C
      pstpbr = DMAX1(W1I,1.0D-35)/dens
C
      RETURN
      END
C  **************************************************************
C                       FUNCTION PLAMBR
C
C  1999-09-09
C
C  Modified from PENELOPE PECALC and PBREM to fit into
C  predpm to get a more accurate brems cross section.
C
C  **************************************************************
      real*8 function plambr(energy)
C
      implicit none
      real*8 energy
C
      real*8 zasum
      common /mater2/ zasum
      real*8 atno,atno2,atno21,mass,dens,excite
      common /mater/ atno,atno2,atno21,mass,dens,excite
      real*8 facion,facbre,facexc,faciom
      common /factor/ facion,facbre,facexc,faciom
      real*8 wcion,wcbre
      common /cutoff/ wcion,wcbre
C
      real*8 mc2,imc2,pi,hrev,a0b,sl,sl2,avog
      parameter (mc2=510.9991d3,imc2=1.d0/mc2)
C
      PARAMETER (PI=3.1415926535897932D0)
      PARAMETER (A0B=5.29177249D-9, HREV=27.2113961D0,
     + SL=137.0359895D0, SL2=SL*SL, AVOG=6.0221367D23)
C
      real*8 fcmtue,fccmie,zzeq,thbh,e,stp
      real*8 FF2,S4I,S0D,S1D,S2D,S1T,S2T
C
      real*8 rstpow
C
C  ****  CONVERSION FACTORS FROM BOHR**2 TO 1/MTU.
C
      FCCMIE=AVOG*dens*A0B**2/(zasum/atno)
      FCMTUE=1.0D-6*AVOG*A0B**2/(zasum/atno)
      ZZEQ=zasum/mass
C
C  cut-off for hard collisions in terms of hmc^2
C
      THBH=wcbre/HREV
C
C  get the radiative stopping power at this energy...
C
      STP=rstpow(energy)*1.d-6
      E=energy/hrev
C
C  get the cross section...
C
      IF(wcbre.GE.10.0D0) THEN
        CALL PEBREM(ZZEQ,THBH,E,S0D,S1D,S2D,S1T,S2T)
        STP=STP/(HREV*FCMTUE)
        FF2=STP/S1T
        S4I=FCCMIE*FF2*S0D
      ELSE
        S4I=0.0D0
      ENDIF
C
C  dpm wants the mfp, not the cross section...
C
      plambr=dens/DMAX1(S4I,1.0D-35)
C
      RETURN
      END
C  **************************************************************
C                       SUBROUTINE PEBREM
C  **************************************************************
C
C  1999-09-09
C  Adapted from PENELOPE...

      SUBROUTINE PEBREM(Z,EC,E,S0D,S1D,S2D,S1,S2)
C
C     CROSS SECTION (PER ATOM) FOR BREMSSTRAHLUNG EMISSION BY
C  FAST ELECTRONS OR POSITRONS. BETHE-HEITLER DIFFERENTIAL CROSS
C  SECTION WITH EXPONENTIAL SCREENING AND COULOMB AND LOW-ENERGY
C  CORRECTIONS.
C
C  INPUT ARGUMENTS:
C     Z ........ ATOMIC NUMBER,
C     EC ....... CUTOFF ENERGY FOR DISCRETE PHOTON EMISSION,
C     E ........ ELECTRON OR POSITRON KINETIC ENERGY,
C
C  OUTPUT PARAMETERS:
C     S0D ...... CROSS SECTION FOR DISCRETE PHOTON EMISSION,
C     S1D ...... STOPPING CROSS SECTION FOR DISCRETE PHOTON
C                EMISSION,
C     S2D ...... STRAGGLING CROSS SECTION FOR DISCRETE PHOTON
C                EMISSION,
C     S1 ....... TOTAL RADIATIVE STOPPING CROSS SECTION,
C     S2 ....... TOTAL RADIATIVE STRAGGLING CROSS SECTION.
C
C                               (ALL QUANTITIES IN ATOMIC UNITS).
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      implicit integer (i-n)
      PARAMETER (SL=137.0359895D0, SL2=SL*SL)
      COMMON/CPAIR/RS(100),ETA(100)
      COMMON/PBREMC/BCOR,GAM,EPCUT,F0(4),BCB
      EXTERNAL DBHSC0,DBHSC1,DBHSC2
C
      IZ=Z+0.25D0
      IF(IZ.LE.0) IZ=1
      IF(IZ.GT.100) IZ=100
C  ****  DBM COULOMB CORRECTION.
      ALZ=Z/SL
      A=ALZ*ALZ
      FC=A*(0.202059D0-A*(0.03693D0-A*(0.00835D0-A*(0.00201D0-A*
     1   (0.00049D0-A*(0.00012D0-A*0.00003)))))+1.0D0/(A+1.0D0))
C  ****  ELECTRON CONTRIBUTION.
      ETAE=((E/SL2)**0.8D0/(2.43D0+(E/SL2)**0.8D0))*ETA(IZ)
C  ****  SCREENING FUNCTIONS.
      BCB=2.0D0/RS(IZ)
      F0(1)=4.0D0*DLOG(RS(IZ))
      F0(2)=F0(1)-4.0D0*FC
C  ****  LOW-ENERGY CORRECTION.
      F0(3)=ALZ
      XX=E/SL2
      F0(4)=(4.650D0-F0(3)*(6.005D0-F0(3)*2.946D0))/XX
     1     -(3.242D1-F0(3)*(6.708D1+F0(3)*3.906D0))/(1+XX)**2
     2     +(2.033D1+F0(3)*(2.338D1-F0(3)*7.742D1))/(1+XX)**3
C  ****  CONSTANTS.
      GAM=1.0D0+E/SL2
      EPCUT=(E-5.0D0*SL2)/(E+SL2)
      IC=1
      IF(EPCUT.GT.0.0D0) IC=2
      EPC=EC/(E+SL2)
      EPM=E/(E+SL2)
      IF(EPM.GT.9.9999D-1) EPM=9.9999D-1
      CALL SCHIFF(0.0D0,F1,F2)
      BCOR=F2+F0(IC)+F0(4)
      BFACT=Z*(Z+ETAE)/SL**5
C
C  ****  INTEGRATED CROSS SECTIONS.
C
      IF(EPM.GT.EPC) THEN
        CALL GABQ(DBHSC0,EPC,EPM,S0D,1.0D-5,IER)
        S0D=S0D+1.333333333333333D0*BCOR*(DLOG(EPM/EPC)-EPM+EPC)
        CALL GABQ(DBHSC1,0.0D0,EPC,S1C,1.0D-5,IER)
        CALL GABQ(DBHSC1,EPC,EPM,S1D,1.0D-5,IER)
        CALL GABQ(DBHSC2,0.0D0,EPC,S2C,1.0D-5,IER)
        CALL GABQ(DBHSC2,EPC,EPM,S2D,1.0D-5,IER)
      ELSE
        S0D=0.0D0
        S1D=0.0D0
        S2D=0.0D0
        CALL GABQ(DBHSC1,0.0D0,EPM,S1C,1.0D-5,IER)
        CALL GABQ(DBHSC2,0.0D0,EPM,S2C,1.0D-5,IER)
      ENDIF
C
      S0D=BFACT*S0D
      S1=BFACT*(S1C+S1D)*(E+SL2)
      S1D=BFACT*S1D*(E+SL2)
      S2=BFACT*(S2C+S2D)*(E+SL2)**2
      S2D=BFACT*S2D*(E+SL2)**2
      RETURN
      END
C  **************************************************************
      FUNCTION DBHSC0(EPS)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      implicit integer (i-n)
      COMMON/PBREMC/BCOR,GAM,EPCUT,F0(4),BCB
      EPS1=1.0D0-EPS
      B=EPS/(BCB*GAM*EPS1)
      IF(EPS.LT.EPCUT) THEN
        IC=2
      ELSE
        IC=1
      ENDIF
      CALL SCHIFF(B,F1,F2)
      DBHSC0=EPS*(F1+F0(IC)+F0(4))
     1      +1.333333333333333D0*EPS1*(F2+F0(IC)+F0(4)-BCOR)/EPS
      RETURN
      END
C  **************************************************************
      FUNCTION DBHSC1(EPS)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      implicit integer (i-n)
      COMMON/PBREMC/BCOR,GAM,EPCUT,F0(4),BCB
      EPS1=1.0D0-EPS
      B=EPS/(BCB*GAM*EPS1)
      IF(EPS.LT.EPCUT) THEN
        IC=2
      ELSE
        IC=1
      ENDIF
      CALL SCHIFF(B,F1,F2)
      DBHSC1=EPS*EPS*(F1+F0(IC)+F0(4))
     1      +1.333333333333333D0*EPS1*(F2+F0(IC)+F0(4))
      RETURN
      END
C  **************************************************************
      FUNCTION DBHSC2(EPS)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      implicit integer (i-n)
      COMMON/PBREMC/BCOR,GAM,EPCUT,F0(4),BCB
      EPS1=1.0D0-EPS
      B=EPS/(BCB*GAM*EPS1)
      IF(EPS.LT.EPCUT) THEN
        IC=2
      ELSE
        IC=1
      ENDIF
      CALL SCHIFF(B,F1,F2)
      DBHSC2=(EPS*EPS*(F1+F0(IC)+F0(4))
     1      +1.333333333333333D0*EPS1*(F2+F0(IC)+F0(4)))*EPS
      RETURN
      END
C  **************************************************************
      SUBROUTINE SCHIFF(B,F1,F2)
C
C     SCREENING FUNCTIONS F1(B) AND F2(B) IN THE BETHE-HEITLER
C  DIFFERENTIAL CROSS SECTION FOR BREMSSTRAHLUNG EMISSION.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      implicit integer (i-n)
      PARAMETER (PI=3.1415926535897932D0, TWOPI=2.0D0*PI)
      B2=B*B
      F1=2.0D0-2.0D0*DLOG(1.0D0+B2)
      F2=F1-6.666666666666666D-1
      IF(B.LT.1.0D-10) THEN
        F1=F1-TWOPI*B
      ELSE
        A0=4.0D0*B*DATAN2(1.0D0,B)
        F1=F1-A0
        F2=F2+2.0D0*B2*(4.0D0-A0-3.0D0*DLOG((1.0D0+B2)/B2))
      ENDIF
      F2=0.5D0*(3.0D0*F1-F2)
      RETURN
      END
