CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C  PPPPP   EEEEEE  N    N   GGGG   EEEEEE   OOOO   M    M   2222
C  P    P  E       NN   N  G    G  E       O    O  MM  MM  2    2
C  P    P  E       N N  N  G       E       O    O  M MM M      2
C  PPPPP   EEEE    N  N N  G  GGG  EEEE    O    O  M    M    22
C  P       E       N   NN  G    G  E       O    O  M    M   2
C  P       EEEEEE  N    N   GGGG   EEEEEE   OOOO   M    M  222222
C
C
C                                           (VERSION 2000.08.28).
C
C   Modular quadric geometry subroutines for Monte Carlo simul-
C   ation (using PENELOPE or detailed procedures) of radiation
C   transport in material systems.
C
C  //////////////////////////////////////////////////////////////
C
C  List of changes/amendments of the original (1998) code:
C
C  **  (F. Salvat. June, 2000) A label ILB was added to common
C  TRACK to provide information about the origin of secondary
C  radiations.
C  --------------------------


C  **************************************************************
C                         SUBROUTINE GEOMIN
C  **************************************************************
      SUBROUTINE GEOMIN(PARINP,NPINP,NMAT,NBOD)
C
C     This subroutine initializes the geometry package for Monte
C  Carlo simulation of particle transport.
C
C  Input arguments:
C     PARINP .... array containing optional parameters, which may
C                 replace the ones entered from the input file.
C     NPINP ..... number of parameters defined in PARINP (.ge.0).
C  Output arguments:
C     NMAT ...... number of different materials in full bodies
C                 (excluding void regions).
C     NBOD ...... Number of defined bodies.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), CHARACTER*8 (L),
     1    INTEGER*4 (I-K,M-N)
      PARAMETER (NS=250,NB=125,NX=100)
      PARAMETER (PI=3.1415926535897932D0)
      PARAMETER (LNUL='00000000',LSUR='SURFACE ',LIND='INDICES=',
     1           LBOD='BODY    ',LMAT='MATERIAL',LMOD='MODULE  ',
     2          LOPEN=')       ',LEND='END     ',LONE='11111111',
     3           LXSC='X-SCALE=',LYSC='Y-SCALE=',LZSC='Z-SCALE=',
     4           LTHE='  THETA=',LPHI='    PHI=',LOME='  OMEGA=',
     5           LXSH='X-SHIFT=',LYSH='Y-SHIFT=',LZSH='Z-SHIFT=',
     6           LAXX='    AXX=',LAXY='    AXY=',LAXZ='    AXZ=',
     7           LAYY='    AYY=',LAYZ='    AYZ=',LAZZ='    AZZ=',
     8            LAX='     AX=', LAY='     AY=', LAZ='     AZ=',
     9            LA0='     A0=',LDEG=') DEG   ',LRAD=') RAD   ')
      DIMENSION PARINP(NPINP),LARRAY(8),KQ(5),KALIAB(NB),KM(NS)
      COMMON/QSURF/AXX(NS),AXY(NS),AXZ(NS),AYY(NS),AYZ(NS),
     1    AZZ(NS),AX(NS),AY(NS),AZ(NS),A0(NS),NSURF,KPLANE(NS)
      COMMON/QTREE/NBODY,MATER(NB),KMOTH(NB),KDGHT(NB,NX),
     1    KSURF(NB,NX),KFLAG(NB,NX),KALIAS(NS),KSLAST
      DATA IR,IW/5,6/
C
C  ************  Initialize parameters.
C
C  ****  Surface coefficients, alias and KPLANE.
C        (KPLANE=1 if the surface is a plane)
      DO KS=1,NS
        KALIAS(KS)=-10000
        AXX(KS)=0.0D0
        AXY(KS)=0.0D0
        AXZ(KS)=0.0D0
        AYY(KS)=0.0D0
        AYZ(KS)=0.0D0
        AZZ(KS)=0.0D0
        AX(KS)=0.0D0
        AY(KS)=0.0D0
        AZ(KS)=0.0D0
        A0(KS)=0.0D0
        KPLANE(KS)=0
      ENDDO
C  ****  Alias, material, mother, daughters, surfaces and side
C        pointers of bodies. The latter component of the double
C        arrays is the number of used components. For instance
C        KSURF(KB,NX) is the number of surfaces that limit the
C        body or module KB.
      DO KB=1,NB
        KALIAB(KB)=-10000
        MATER(KB)=0
        KMOTH(KB)=0
        DO K2=1,NX
          KDGHT(KB,K2)=0
          KSURF(KB,K2)=0
          KFLAG(KB,K2)=5
        ENDDO
      ENDDO
      NMAT=0
      NSURF=0
      NBODY=0
C
C  ************  Reading the geometry input file.
C
    1 CONTINUE
      READ(IR,1000) LLL,(LARRAY(I),I=1,7)
 1000 FORMAT(8A8)
      IF(LLL.NE.LNUL) THEN
        WRITE(IW,2000) LLL,(LARRAY(I),I=1,7)
        GO TO 1
      ELSE
        WRITE(IW,2001)
      ENDIF
 2000 FORMAT(8A8)
 2001 FORMAT(64('0'))
C
    2 CONTINUE
      READ(IR,1001) LLL,IT,(LARRAY(I),I=1,7)
 1001 FORMAT(A8,1X,I3,7A8)
      IF(LLL.EQ.LSUR) GO TO 100
      IF(LLL.EQ.LBOD) GO TO 200
      IF(LLL.EQ.LMOD) GO TO 300
      IF(LLL.EQ.LEND) GO TO 400
      WRITE(IW,2002) LLL,IT,(LARRAY(I),I=1,7)
 2002 FORMAT(A8,'(',I3,7A8)
      WRITE(IW,3000)
 3000 FORMAT('***  WHAT DO YOU MEAN?')
      STOP
C
C  ************  Surfaces.
C
  100 CONTINUE
      IF(NSURF.GT.0) THEN
        DO KS0=1,NSURF
          IF(IT.EQ.KALIAS(KS0)) THEN
            WRITE(IW,2002) LLL,IT,(LARRAY(I),I=1,7)
            WRITE(IW,3001)
            STOP
          ENDIF
        ENDDO
      ENDIF
 3001 FORMAT('***  SAME LABEL FOR TWO SURFACES.')
      NSURF=NSURF+1
      KS=NSURF
      WRITE(IW,2002) LLL,KS,(LARRAY(I),I=1,7)
      IF(KS.GT.NS) THEN
        WRITE(IW,3002)
        STOP
      ENDIF
 3002 FORMAT('***  THE PARAMETER NS MUST BE INCREASED.')
      KALIAS(KS)=IT
C  ****  Indices.
      READ(IR,1002) LLL,(KQ(I),I=1,5)
 1002 FORMAT(A8,1X,I2,1X,I2,1X,I2,1X,I2,1X,I2)
      WRITE(IW,2003) LLL,(KQ(I),I=1,5)
 2003 FORMAT(A8,'(',4(I2,','),I2,')')
      IMODE=MAX0(IABS(KQ(1)),IABS(KQ(2)),IABS(KQ(3)),IABS(KQ(4)),
     1      IABS(KQ(5)))
      IF(IMODE.GT.1.OR.LLL.NE.LIND) THEN
        WRITE(6,3003)
        STOP
      ENDIF
 3003 FORMAT('***  INCORRECT SURFACE INDICES.')
      IF(IMODE.EQ.0) GO TO 103
C
C  ****  Reduced form.
C
      XSCALE=1.0D0
      YSCALE=1.0D0
      ZSCALE=1.0D0
      OMEGA=0.0D0
      THETA=0.0D0
      PHI=0.0D0
      XSHIFT=0.0D0
      YSHIFT=0.0D0
      ZSHIFT=0.0D0
C  ****  Quadric parameters.
  101 CONTINUE
      READ(IR,1003) LLL,VALUE,ICHANG,LANGLE
 1003 FORMAT(A8,1X,E22.15,1X,I3,A8)
      IF(LLL.EQ.LNUL) GO TO 102
      IF(ICHANG.LE.NPINP) THEN
        IF(ICHANG.GT.0) THEN
          VALUE=PARINP(ICHANG)
          ICHANG=-ICHANG
        ENDIF
      ELSE
        WRITE(IW,2004) LLL,VALUE,ICHANG,LRAD
        WRITE(IW,3004)
        STOP
      ENDIF
 2004 FORMAT(A8,'(',1P,E22.15,',',I3,A8)
 3004 FORMAT('***  NPINP IS TOO SMALL (CHECK PARINP).')
      IF(LLL.EQ.LXSC) THEN
        XSCALE=VALUE
        WRITE(IW,2004) LLL,VALUE,ICHANG,LOPEN
        IF(VALUE.LT.1.0D-15) STOP
      ELSE IF(LLL.EQ.LYSC) THEN
        YSCALE=VALUE
        WRITE(IW,2004) LLL,VALUE,ICHANG,LOPEN
        IF(VALUE.LT.1.0D-15) STOP
      ELSE IF(LLL.EQ.LZSC) THEN
        ZSCALE=VALUE
        WRITE(IW,2004) LLL,VALUE,ICHANG,LOPEN
        IF(VALUE.LT.1.0D-15) STOP
      ELSE IF(LLL.EQ.LOME) THEN
        IF(LANGLE.EQ.LRAD) THEN
          OMEGA=VALUE
          WRITE(IW,2004) LLL,VALUE,ICHANG,LRAD
        ELSE
          OMEGA=VALUE*PI/180.0D0
          WRITE(IW,2004) LLL,VALUE,ICHANG,LDEG
        ENDIF
      ELSE IF(LLL.EQ.LTHE) THEN
        IF(LANGLE.EQ.LRAD) THEN
          THETA=VALUE
          WRITE(IW,2004) LLL,VALUE,ICHANG,LRAD
        ELSE
          THETA=VALUE*PI/180.0D0
          WRITE(IW,2004) LLL,VALUE,ICHANG,LDEG
        ENDIF
      ELSE IF(LLL.EQ.LPHI) THEN
        IF(LANGLE.EQ.LRAD) THEN
          PHI=VALUE
          WRITE(IW,2004) LLL,VALUE,ICHANG,LRAD
        ELSE
          PHI=VALUE*PI/180.0D0
          WRITE(IW,2004) LLL,VALUE,ICHANG,LDEG
        ENDIF
      ELSE IF(LLL.EQ.LXSH) THEN
        XSHIFT=VALUE
        WRITE(IW,2004) LLL,VALUE,ICHANG,LOPEN
      ELSE IF(LLL.EQ.LYSH) THEN
        YSHIFT=VALUE
        WRITE(IW,2004) LLL,VALUE,ICHANG,LOPEN
      ELSE IF(LLL.EQ.LZSH) THEN
        ZSHIFT=VALUE
        WRITE(IW,2004) LLL,VALUE,ICHANG,LOPEN
      ELSE
        WRITE(IW,2004) LLL,VALUE,ICHANG,LANGLE
        WRITE(IW,3000)
        STOP
      ENDIF
      GO TO 101
C  ****  Expanded quadric.
  102 CONTINUE
      QXX=KQ(1)/XSCALE**2
      QXY=0.0D0
      QXZ=0.0D0
      QYY=KQ(2)/YSCALE**2
      QYZ=0.0D0
      QZZ=KQ(3)/ZSCALE**2
      QX=0.0D0
      QY=0.0D0
      QZ=KQ(4)/ZSCALE
      Q0=KQ(5)
      CALL ROTSHF(OMEGA,THETA,PHI,XSHIFT,YSHIFT,ZSHIFT,
     1            QXX,QXY,QXZ,QYY,QYZ,QZZ,QX,QY,QZ,Q0)
      AXX(KS)=QXX
      AXY(KS)=QXY
      AXZ(KS)=QXZ
      AYY(KS)=QYY
      AYZ(KS)=QYZ
      AZZ(KS)=QZZ
      AX(KS)=QX
      AY(KS)=QY
      AZ(KS)=QZ
      A0(KS)=Q0
      IF(DMIN1(AXX(KS),AXY(KS),AXZ(KS),AYY(KS),AYZ(KS),AZZ(KS))
     1 .GT.-1.0D-30.AND.
     2   DMAX1(AXX(KS),AXY(KS),AXZ(KS),AYY(KS),AYZ(KS),AZZ(KS))
     3 .LT.1.0D-30) KPLANE(KS)=1
C
      WRITE(IW,2001)
      GO TO 2
C
C  ****  Implicit form.
C
  103 CONTINUE
      READ(IR,1003) LLL,VALUE,ICHANG,LANGLE
      IF(LLL.EQ.LNUL) GO TO 104
      IF(ICHANG.LE.NPINP) THEN
        IF(ICHANG.GT.0) THEN
          VALUE=PARINP(ICHANG)
          ICHANG=-ICHANG
        ENDIF
      ELSE
        WRITE(IW,2004) LLL,VALUE,ICHANG,LRAD
        WRITE(IW,3004)
        STOP
      ENDIF
      IF(LLL.EQ.LAXX) THEN
        AXX(KS)=VALUE
        WRITE(IW,2004) LAXX,AXX(KS),ICHANG,LOPEN
      ELSE IF(LLL.EQ.LAXY) THEN
        AXY(KS)=VALUE
        WRITE(IW,2004) LAXY,AXY(KS),ICHANG,LOPEN
      ELSE IF(LLL.EQ.LAXZ) THEN
        AXZ(KS)=VALUE
        WRITE(IW,2004) LAXZ,AXZ(KS),ICHANG,LOPEN
      ELSE IF(LLL.EQ.LAYY) THEN
        AYY(KS)=VALUE
        WRITE(IW,2004) LAYY,AYY(KS),ICHANG,LOPEN
      ELSE IF(LLL.EQ.LAYZ) THEN
        AYZ(KS)=VALUE
        WRITE(IW,2004) LAYZ,AYZ(KS),ICHANG,LOPEN
      ELSE IF(LLL.EQ.LAZZ) THEN
        AZZ(KS)=VALUE
        WRITE(IW,2004) LAZZ,AZZ(KS),ICHANG,LOPEN
      ELSE IF(LLL.EQ.LAX) THEN
        AX(KS)=VALUE
        WRITE(IW,2004) LAX,AX(KS),ICHANG,LOPEN
      ELSE IF(LLL.EQ.LAY) THEN
        AY(KS)=VALUE
        WRITE(IW,2004) LAY,AY(KS),ICHANG,LOPEN
      ELSE IF(LLL.EQ.LAZ) THEN
        AZ(KS)=VALUE
        WRITE(IW,2004) LAZ,AZ(KS),ICHANG,LOPEN
      ELSE IF(LLL.EQ.LA0) THEN
        A0(KS)=VALUE
        WRITE(IW,2004) LA0,A0(KS),ICHANG,LOPEN
      ELSE
        WRITE(IW,2004) LAXX,AXX(KS),ICHANG,LANGLE
        WRITE(IW,3000)
        STOP
      ENDIF
      GO TO 103
  104 CONTINUE
      IF(DMIN1(AXX(KS),AXY(KS),AXZ(KS),AYY(KS),AYZ(KS),AZZ(KS))
     1 .GT.-1.0D-30.AND.
     2   DMAX1(AXX(KS),AXY(KS),AXZ(KS),AYY(KS),AYZ(KS),AZZ(KS))
     3 .LT.1.0D-30) KPLANE(KS)=1
      WRITE(IW,2001)
      GO TO 2
C
C  ************  Bodies.
C
  200 CONTINUE
      IF(NBODY.GT.0) THEN
        DO KB0=1,NBODY
          IF(IT.EQ.KALIAB(KB0)) THEN
            WRITE(IW,2002) LLL,IT,(LARRAY(I),I=1,7)
            WRITE(IW,3005)
            STOP
          ENDIF
        ENDDO
      ENDIF
 3005 FORMAT('***  SAME LABEL FOR TWO BODIES (OR MODULES).')
      NBODY=NBODY+1
      WRITE(IW,2002) LLL,NBODY,(LARRAY(I),I=1,7)
      IF(NBODY.GT.NB) THEN
        WRITE(IW,3006)
        STOP
      ENDIF
 3006 FORMAT('***  THE PARAMETER NB MUST BE INCREASED.')
      KALIAB(NBODY)=IT
C
      READ(IR,1004) LLL,IMAT
 1004 FORMAT(A8,1X,I3)
      WRITE(IW,2005) LLL,IMAT
 2005 FORMAT(A8,'(',I3,')')
      IF(LLL.NE.LMAT) THEN
        WRITE(IW,3007)
        STOP
      ENDIF
 3007 FORMAT('***  INCORRECT MATERIAL DEFINITION LINE.')
      IF(IMAT.LT.0) IMAT=0
      MATER(NBODY)=IMAT
      NMAT=MAX0(NMAT,IMAT)
C
  201 CONTINUE
      READ(IR,1005) LLL,KIN,INDS
 1005 FORMAT(A8,1X,I3,17X,I2)
      IF(LLL.EQ.LNUL) GO TO 206
      IF(LLL.EQ.LSUR) THEN
C  ****  Surface.
        DO KS0=1,NSURF
          IF(KIN.EQ.KALIAS(KS0)) THEN
            KS=KS0
            GO TO 202
          ENDIF
        ENDDO
        WRITE(IW,2006) LLL,KIN,INDS
        WRITE(IW,3008)
        STOP
  202   CONTINUE
 3008   FORMAT('***  UNDEFINED SURFACE LABEL.')
        WRITE(IW,2006) LLL,KS,INDS
 2006   FORMAT(A8,'(',I3,'), SIDE POINTER=(',I2,')')
C
        KST=KSURF(NBODY,NX)
        IF(KST.GT.0) THEN
          KST0=KST
          DO K=1,KST0
            IF(KS.EQ.KSURF(NBODY,K)) THEN
              IF(KFLAG(NBODY,K).LT.3) THEN
                WRITE(IW,3009)
                STOP
              ELSE
                KST=K
                GO TO 203
              ENDIF
            ENDIF
          ENDDO
        ENDIF
 3009 FORMAT('***  THE LAST LIMITING SURFACE HAS BEEN DEFIN',
     1  'ED TWICE.')
        KST=KST+1
        IF(KST.GE.NX) THEN
          WRITE(IW,3010)
          STOP
        ENDIF
 3010   FORMAT('***  THE PARAMETER NX IS TOO SMALL.')
        KSURF(NBODY,NX)=KST
        KSURF(NBODY,KST)=KS
  203   CONTINUE
        IF(INDS.EQ.-1) THEN
          KFLAG(NBODY,KST)=1
        ELSE IF(INDS.EQ.1) THEN
          KFLAG(NBODY,KST)=2
        ELSE
          WRITE(IW,3011)
          STOP
        ENDIF
 3011   FORMAT('***  CHECK SIDE POINTER VALUE.')
      ELSE IF(LLL.EQ.LBOD) THEN
C  ****  Body.
        IF(NBODY.GT.1) THEN
          DO KB0=1,NBODY-1
            IF(KIN.EQ.KALIAB(KB0)) THEN
              KB=KB0
              GO TO 204
            ENDIF
          ENDDO
          WRITE(IW,2007) LLL,KIN
          WRITE(IW,3012)
          STOP
        ENDIF
  204   CONTINUE
 3012   FORMAT('***  UNDEFINED BODY LABEL.')
        WRITE(IW,2007) LLL,KB
 2007   FORMAT(A8,'(',I3,')')
C
        KN1=KSURF(KB,NX)
        DO 205 KS1=1,KN1
          KSURF1=KSURF(KB,KS1)
          KN2=KSURF(NBODY,NX)
          DO KS2=1,KN2
            KSURF2=KSURF(NBODY,KS2)
            IF(KSURF2.EQ.KSURF1) GO TO 205
          ENDDO
          KST=KN2+1
          IF(KST.GE.NX) THEN
            WRITE(IW,3010)
            STOP
          ENDIF
          KSURF(NBODY,NX)=KST
          KSURF(NBODY,KST)=KSURF1
          KFLAG(NBODY,KST)=3
  205   CONTINUE
      ELSE
        WRITE(IW,2006) LLL,KIN,INDS
        WRITE(IW,3000)
        STOP
      ENDIF
      GO TO 201
  206 CONTINUE
      WRITE(IW,2001)
      GO TO 2
C
C  ************  Modules.
C
  300 CONTINUE
      IF(NBODY.GT.0) THEN
        DO KB0=1,NBODY
        IF(IT.EQ.KALIAB(KB0)) THEN
          WRITE(IW,2002) LLL,IT,(LARRAY(I),I=1,7)
          WRITE(IW,3005)
          STOP
        ENDIF
        ENDDO
      ENDIF
      NBODY=NBODY+1
      WRITE(IW,2002) LLL,NBODY,(LARRAY(I),I=1,7)
      IF(NBODY.GT.NB) THEN
        WRITE(IW,3006)
        STOP
      ENDIF
      KALIAB(NBODY)=IT
C
      READ(IR,1004) LLL,IMAT
      WRITE(IW,2005) LLL,IMAT
      IF(LLL.NE.LMAT) THEN
        WRITE(IW,3007)
        STOP
      ENDIF
      IF(IMAT.LT.0) IMAT=0
      MATER(NBODY)=IMAT
      NMAT=MAX0(NMAT,IMAT)
C
      KDGHT(NBODY,NX)=1
      KDGHT(NBODY,1)=NBODY
C
C  ****  Limiting surfaces and components.
C
  301 CONTINUE
      READ(IR,1005) LLL,KIN,INDS
      IF(LLL.EQ.LNUL.OR.LLL.EQ.LONE) THEN
        KDT=KDGHT(NBODY,NX)
C  ****  Sort daughters in increasing order.
        IF(KDT.GT.1) THEN
          DO KI=1,KDT-1
            KBMIN=KDGHT(NBODY,KI)
            KMIN=KI
            DO KJ=KI+1,KDT
              IF(KDGHT(NBODY,KJ).LT.KBMIN) THEN
                KBMIN=KDGHT(NBODY,KJ)
                KMIN=KJ
              ENDIF
            ENDDO
            IF(KMIN.NE.KI) THEN
              KSAVE=KDGHT(NBODY,KI)
              KDGHT(NBODY,KI)=KDGHT(NBODY,KMIN)
              KDGHT(NBODY,KMIN)=KSAVE
            ENDIF
          ENDDO
        ENDIF
      ENDIF
      IF(LLL.EQ.LONE) GO TO 308
      IF(LLL.EQ.LNUL) GO TO 312
      IF(LLL.EQ.LSUR) THEN
C  ****  Limiting surface.
        DO KS0=1,NSURF
          IF(KIN.EQ.KALIAS(KS0)) THEN
            KS=KS0
            GO TO 302
          ENDIF
        ENDDO
        WRITE(IW,2006) LLL,KIN,INDS
        WRITE(IW,3008)
        STOP
  302   CONTINUE
        WRITE(IW,2006) LLL,KS,INDS
C
        KST=KSURF(NBODY,NX)
        IF(KST.GT.0) THEN
          KST0=KST
          DO K=1,KST0
            IF(KS.EQ.KSURF(NBODY,K)) THEN
              IF(KFLAG(NBODY,K).LT.3) THEN
                WRITE(IW,3009)
                STOP
              ELSE
                KST=K
                GO TO 303
              ENDIF
            ENDIF
          ENDDO
        ENDIF
        KST=KST+1
        IF(KST.GE.NX) THEN
          WRITE(IW,3010)
          STOP
        ENDIF
        KSURF(NBODY,NX)=KST
        KSURF(NBODY,KST)=KS
  303   CONTINUE
        IF(INDS.EQ.-1) THEN
          KFLAG(NBODY,KST)=1
        ELSE IF(INDS.EQ.1) THEN
          KFLAG(NBODY,KST)=2
        ELSE
          WRITE(IW,3011)
          STOP
        ENDIF
C  ****  Body.
      ELSE IF(LLL.EQ.LBOD) THEN
        IF(NBODY.GT.1) THEN
          DO KB0=1,NBODY-1
            IF(KIN.EQ.KALIAB(KB0)) THEN
              KB=KB0
              GO TO 304
            ENDIF
          ENDDO
          WRITE(IW,2007) LLL,KIN
          WRITE(IW,3012)
          STOP
        ENDIF
  304   CONTINUE
        WRITE(IW,2007) LLL,KB
        IF(KMOTH(KB).GT.0) THEN
          WRITE(IW,3013)
 3013   FORMAT('***  YOU ARE TRYING TO GIVE TWO MOTHERS TO',
     1   ' THE LAST BODY.')
          STOP
        ENDIF
        GO TO 306
C  ****  Module.
      ELSE IF(LLL.EQ.LMOD) THEN
        IF(NBODY.GT.0) THEN
          DO KB0=1,NBODY-1
            IF(KIN.EQ.KALIAB(KB0)) THEN
              KB=KB0
              GO TO 305
            ENDIF
          ENDDO
          WRITE(IW,2007) LLL,KIN
          WRITE(IW,3012)
          STOP
        ENDIF
  305   CONTINUE
        WRITE(IW,2007) LLL,KB
        IF(KMOTH(KB).GT.0) THEN
          WRITE(IW,3014)
 3014   FORMAT('***  YOU ARE TRYING TO GIVE TWO MOTHERS TO',
     1   ' THE LAST MODULE.')
          STOP
        ENDIF
        GO TO 306
C
      ELSE
        WRITE(IW,2006) LLL,KIN,INDS
        WRITE(IW,3000)
        STOP
      ENDIF
      GO TO 301
C
  306 CONTINUE
      KMOTH(KB)=NBODY
      KDT=KDGHT(NBODY,NX)+1
      KDGHT(NBODY,NX)=KDT
      KDGHT(NBODY,KDT)=KB
C
      KN1=KSURF(KB,NX)
      DO 307 KS1=1,KN1
        IF(KFLAG(KB,KS1).GT.3) GO TO 307
        KSURF1=KSURF(KB,KS1)
        KN2=KSURF(NBODY,NX)
        DO KS2=1,KN2
          KSURF2=KSURF(NBODY,KS2)
          IF(KSURF2.EQ.KSURF1) GO TO 307
        ENDDO
        KN2=KN2+1
        IF(KN2.GE.NX) THEN
          WRITE(IW,3010)
          STOP
        ENDIF
        KSURF(NBODY,NX)=KN2
        KSURF(NBODY,KN2)=KSURF1
        KFLAG(NBODY,KN2)=4
  307 CONTINUE
      GO TO 301
C
C  ****  Transformation parameters.
C
  308 CONTINUE
      WRITE(IW,2008)
 2008 FORMAT(64('1'))
      OMEGA=0.0D0
      THETA=0.0D0
      PHI=0.0D0
      XSHIFT=0.0D0
      YSHIFT=0.0D0
      ZSHIFT=0.0D0
C
  309 CONTINUE
      READ(IR,1003) LLL,VALUE,ICHANG,LANGLE
      IF(LLL.EQ.LNUL) GO TO 310
      IF(ICHANG.LE.NPINP) THEN
        IF(ICHANG.GT.0) THEN
          VALUE=PARINP(ICHANG)
          ICHANG=-ICHANG
        ENDIF
      ELSE
        WRITE(IW,2004) LLL,VALUE,ICHANG,LRAD
        WRITE(IW,3004)
        STOP
      ENDIF
      IF(LLL.EQ.LOME) THEN
        IF(LANGLE.EQ.LRAD) THEN
          OMEGA=VALUE
          WRITE(IW,2004) LLL,VALUE,ICHANG,LRAD
        ELSE
          OMEGA=VALUE*PI/180.0D0
          WRITE(IW,2004) LLL,VALUE,ICHANG,LDEG
        ENDIF
      ELSE IF(LLL.EQ.LTHE) THEN
        IF(LANGLE.EQ.LRAD) THEN
          THETA=VALUE
          WRITE(IW,2004) LLL,VALUE,ICHANG,LRAD
        ELSE
          THETA=VALUE*PI/180.0D0
          WRITE(IW,2004) LLL,VALUE,ICHANG,LDEG
        ENDIF
      ELSE IF(LLL.EQ.LPHI) THEN
        IF(LANGLE.EQ.LRAD) THEN
          PHI=VALUE
          WRITE(IW,2004) LLL,VALUE,ICHANG,LRAD
        ELSE
          PHI=VALUE*PI/180.0D0
          WRITE(IW,2004) LLL,VALUE,ICHANG,LDEG
        ENDIF
      ELSE IF(LLL.EQ.LXSH) THEN
        XSHIFT=VALUE
        WRITE(IW,2004) LLL,VALUE,ICHANG,LOPEN
      ELSE IF(LLL.EQ.LYSH) THEN
        YSHIFT=VALUE
        WRITE(IW,2004) LLL,VALUE,ICHANG,LOPEN
      ELSE IF(LLL.EQ.LZSH) THEN
        ZSHIFT=VALUE
        WRITE(IW,2004) LLL,VALUE,ICHANG,LOPEN
      ELSE
        WRITE(IW,2004) LLL,VALUE,ICHANG,LANGLE
        WRITE(IW,3000)
        STOP
      ENDIF
      GO TO 309
C
C  ****  Rotation and translation of the module (its surfaces).
C
  310 CONTINUE
      DO KS=1,NSURF
        KM(KS)=0
      ENDDO
      DO KB=1,NBODY
        KMB=KB
  311   CONTINUE
        IF(KMB.EQ.NBODY) THEN
          KNS=KSURF(KB,NX)
          DO KSS=1,KNS
            KS=KSURF(KB,KSS)
            IF(KFLAG(KB,KSS).LT.5.AND.KM(KS).EQ.0) THEN
              QXX=AXX(KS)
              QXY=AXY(KS)
              QXZ=AXZ(KS)
              QYY=AYY(KS)
              QYZ=AYZ(KS)
              QZZ=AZZ(KS)
              QX=AX(KS)
              QY=AY(KS)
              QZ=AZ(KS)
              Q0=A0(KS)
              CALL ROTSHF(OMEGA,THETA,PHI,XSHIFT,YSHIFT,ZSHIFT,
     1                    QXX,QXY,QXZ,QYY,QYZ,QZZ,QX,QY,QZ,Q0)
              AXX(KS)=QXX
              AXY(KS)=QXY
              AXZ(KS)=QXZ
              AYY(KS)=QYY
              AYZ(KS)=QYZ
              AZZ(KS)=QZZ
              AX(KS)=QX
              AY(KS)=QY
              AZ(KS)=QZ
              A0(KS)=Q0
              KM(KS)=1
            ENDIF
          ENDDO
        ELSE
         KMB=KMOTH(KMB)
         IF(KMB.GT.0) GO TO 311
        ENDIF
      ENDDO
C
  312 CONTINUE
      WRITE(IW,2001)
      GO TO 2
C
C  ************  End-line in the input file.
C
  400 CONTINUE
      IF(NBODY.EQ.1) THEN
        KDGHT(1,NX)=1
        KDGHT(1,1)=1
      ENDIF
C
C  ****  Define the enclosure, if it has not been defined yet.
C
      NORPH=0
      DO KB=1,NBODY
        IF(KMOTH(KB).EQ.0) NORPH=NORPH+1
      ENDDO
C  ****  The next line serves only to avoid a warning message
C        issued by certain compilers.
      NT=0
      IF(NORPH.GT.1) THEN
        KS=NSURF+1
        AXX(KS)=1.0D0
        AYY(KS)=1.0D0
        AZZ(KS)=1.0D0
        A0(KS)=-1.0D30
        KB=NBODY+1
        KSURF(KB,NX)=1
        KSURF(KB,1)=KS
        KFLAG(KB,1)=1
        KDGHT(KB,NX)=1
        KDGHT(KB,1)=KB
        DO KBB=1,NBODY
          IF(KMOTH(KBB).EQ.0) THEN
            NT=KDGHT(KB,NX)+1
            KDGHT(KB,NX)=NT
            KDGHT(KB,NT)=KBB
            KN1=KSURF(KBB,NX)
            DO 401 KS1=1,KN1
              IF(KFLAG(KBB,KS1).GT.3) GO TO 401
              KSURF1=KSURF(KBB,KS1)
              KN2=KSURF(KB,NX)
              DO KS2=1,KN2
                IF(KSURF(KB,KS2).EQ.KSURF1) GO TO 401
              ENDDO
              KN2=KN2+1
              IF(KN2.GE.NX) THEN
                WRITE(IW,3010)
                STOP
              ENDIF
              KSURF(KB,NX)=KN2
              KSURF(KB,KN2)=KSURF1
              KFLAG(KB,KN2)=4
  401       CONTINUE
            KMOTH(KBB)=KB
          ENDIF
        ENDDO
        NSURF=KS
        NBODY=KB
        IF(NBODY.GT.NB) THEN
          WRITE(IW,3006)
          STOP
        ENDIF
        IF(NSURF.GT.NS) THEN
          WRITE(IW,3002)
          STOP
        ENDIF
C  ****  Sort daughters in increasing order.
        IF(NT.GT.1) THEN
          DO KI=1,NT-1
            KBMIN=KDGHT(NBODY,KI)
            KMIN=KI
            DO KJ=KI+1,NT
              IF(KDGHT(NBODY,KJ).LT.KBMIN) THEN
                KBMIN=KDGHT(NBODY,KJ)
                KMIN=KJ
              ENDIF
            ENDDO
            IF(KMIN.NE.KI) THEN
              KSAVE=KDGHT(NBODY,KI)
              KDGHT(NBODY,KI)=KDGHT(NBODY,KMIN)
              KDGHT(NBODY,KMIN)=KSAVE
            ENDIF
          ENDDO
        ENDIF
      ENDIF
C
      WRITE(IW,2009) LLL
 2009 FORMAT(A8,1X,55('0'))
      NBOD=NBODY
C
      RETURN
      END
C  **************************************************************
C                         SUBROUTINE ROTSHF
C  **************************************************************
      SUBROUTINE ROTSHF(OMEGA,THETA,PHI,DX,DY,DZ,
     1                  AXX,AXY,AXZ,AYY,AYZ,AZZ,AX,AY,AZ,A0)
C
C     This subroutine rotates and shifts a quadric surface.
C
C  Input parameters:
C     OMEGA, THETA, PHI ... Euler rotation angles,
C     DX, DY, DZ .......... components of the displacement
C                           vector,
C     AXX, ..., A0 ........ coefficients of the initial quadric.
C
C  Output parameters:
C     AXX, ..., A0 ........ coefficients of the transformed
C                           quadric surface.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      DIMENSION R(3,3),A2(3,3),B2(3,3),A1(3),B1(3),D1(3)
C
C  ****  Initial quadric.
C
      B2(1,1)=AXX
      B2(1,2)=0.5D0*AXY
      B2(1,3)=0.5D0*AXZ
      B2(2,1)=B2(1,2)
      B2(2,2)=AYY
      B2(2,3)=0.5D0*AYZ
      B2(3,1)=B2(1,3)
      B2(3,2)=B2(2,3)
      B2(3,3)=AZZ
      B1(1)=AX
      B1(2)=AY
      B1(3)=AZ
      B0=A0
      D1(1)=DX
      D1(2)=DY
      D1(3)=DZ
C
C  ****  Rotation matrix.
C
      STHETA=DSIN(THETA)
      CTHETA=DCOS(THETA)
      SPHI=DSIN(PHI)
      CPHI=DCOS(PHI)
      SOMEGA=DSIN(OMEGA)
      COMEGA=DCOS(OMEGA)
C
      R(1,1)=CPHI*CTHETA*COMEGA-SPHI*SOMEGA
      R(1,2)=-CPHI*CTHETA*SOMEGA-SPHI*COMEGA
      R(1,3)=CPHI*STHETA
      R(2,1)=SPHI*CTHETA*COMEGA+CPHI*SOMEGA
      R(2,2)=-SPHI*CTHETA*SOMEGA+CPHI*COMEGA
      R(2,3)=SPHI*STHETA
      R(3,1)=-STHETA*COMEGA
      R(3,2)=STHETA*SOMEGA
      R(3,3)=CTHETA
C
C  ****  Rotated quadric.
C
      DO I=1,3
        A1(I)=0.0D0
        DO J=1,3
          A1(I)=A1(I)+R(I,J)*B1(J)
          A2(I,J)=0.0D0
          DO M=1,3
            DO K=1,3
              A2(I,J)=A2(I,J)+R(I,K)*B2(K,M)*R(J,M)
            ENDDO
          ENDDO
        ENDDO
      ENDDO
C
C  ****  Shifted-rotated quadric.
C
      DO I=1,3
        A2D=0.0D0
        DO J=1,3
          A2D=A2D+A2(I,J)*D1(J)
        ENDDO
        B1(I)=A1(I)-2.0D0*A2D
        B0=B0+D1(I)*(A2D-A1(I))
      ENDDO
C
      AXX=A2(1,1)
      IF(DABS(AXX).LT.1.0D-15) AXX=0.0D0
      AXY=A2(1,2)+A2(2,1)
      IF(DABS(AXY).LT.1.0D-15) AXY=0.0D0
      AXZ=A2(1,3)+A2(3,1)
      IF(DABS(AXZ).LT.1.0D-15) AXZ=0.0D0
      AYY=A2(2,2)
      IF(DABS(AYY).LT.1.0D-15) AYY=0.0D0
      AYZ=A2(2,3)+A2(3,2)
      IF(DABS(AYZ).LT.1.0D-15) AYZ=0.0D0
      AZZ=A2(3,3)
      IF(DABS(AZZ).LT.1.0D-15) AZZ=0.0D0
      AX=B1(1)
      IF(DABS(AX).LT.1.0D-15) AX=0.0D0
      AY=B1(2)
      IF(DABS(AY).LT.1.0D-15) AY=0.0D0
      AZ=B1(3)
      IF(DABS(AZ).LT.1.0D-15) AZ=0.0D0
      A0=B0
      IF(DABS(A0).LT.1.0D-15) A0=0.0D0
      RETURN
      END
C  **************************************************************
C                       SUBROUTINE LOCATE
C  **************************************************************
      SUBROUTINE LOCATE
C
C     This subroutine determines the body that contains the point
C  with coordinates (X,Y,Z).
C
C  Input values (common /TRACK/):
C     X, Y, Z ... coordinates of the particle,
C     U, V, W ... direction of movement.
C
C  Output values (common /TRACK/):
C     IBODY ..... body where the particle moves,
C     MAT  ...... material in ibody,
C                    MAT=0, indicates a void region.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      PARAMETER (NS=250,NB=125,NX=100)
      PARAMETER(EPS=1.0D-8)
      COMMON/QSURF/AXX(NS),AXY(NS),AXZ(NS),AYY(NS),AYZ(NS),
     1    AZZ(NS),AX(NS),AY(NS),AZ(NS),A0(NS),NSURF,KPLANE(NS)
      COMMON/QTREE/NBODY,MATER(NB),KMOTH(NB),KDGHT(NB,NX),
     1    KSURF(NB,NX),KFLAG(NB,NX),KSP(NS),KSLAST
C
      COMMON/TRACK/E,X,Y,Z,U,V,W,WGHT,KPAR,IBODY,MAT,ILB
C
      DO KS=1,NSURF
        KSP(KS)=0
      ENDDO
      KB0=NBODY
  100 CONTINUE
      NST=KSURF(KB0,NX)
      DO 1 KSS=1,NST
        KS=KSURF(KB0,KSS)
        IF(KSP(KS).NE.0) GO TO 1
        IF(KPLANE(KS).EQ.0) THEN
          F=X*(AXX(KS)*X+AXY(KS)*Y+AXZ(KS)*Z+AX(KS))
     1     +Y*(AYY(KS)*Y+AYZ(KS)*Z+AY(KS))
     2     +Z*(AZZ(KS)*Z+AZ(KS))+A0(KS)
        ELSE
          F=X*AX(KS)+Y*AY(KS)+Z*AZ(KS)+A0(KS)
        ENDIF
C
C  ****  Point close to a surface.
C
        IF(DABS(F).LT.1.0D-12) THEN
C  ****  Quadric surface gradient.
          DFX=2.0D0*AXX(KS)*X+AXY(KS)*Y+AXZ(KS)*Z+AX(KS)
          DFY=AXY(KS)*X+2.0D0*AYY(KS)*Y+AYZ(KS)*Z+AY(KS)
          DFZ=AXZ(KS)*X+AYZ(KS)*Y+2.0D0*AZZ(KS)*Z+AZ(KS)
          DF=DSQRT(DFX*DFX+DFY*DFY+DFZ*DFZ)
          DOTP=U*DFX+V*DFY+W*DFZ
          IF(DOTP.GT.0.0D0) THEN
            ISN=1
          ELSE
            ISN=-1
          ENDIF
C  ****  Shift the point a small distance along the gradient
C        direction.
          IF(DF.GT.1.0D-12) THEN
            X=X+ISN*EPS*DFX/DF
            Y=Y+ISN*EPS*DFY/DF
            Z=Z+ISN*EPS*DFZ/DF
          ELSE
            X=X+ISN*EPS
            Y=Y+ISN*EPS
            Z=Z+ISN*EPS
          ENDIF
          DO KST=1,NSURF
            KSP(KST)=0
          ENDDO
          KB0=NBODY
          GO TO 100
        ENDIF
C
        IF(F.LT.0.0D0) THEN
          KSP(KS)=1
        ELSE
          KSP(KS)=2
        ENDIF
    1 CONTINUE
C
C  ****  Determine the module or body that contains the point.
C
      NDT=KDGHT(KB0,NX)
      DO 2 KBB=1,NDT
        KB=KDGHT(KB0,KBB)
        NST=KSURF(KB,NX)
        DO KSS=1,NST
          KS=KSURF(KB,KSS)
          KF=KFLAG(KB,KSS)
          IF(KF.LT.3.AND.KSP(KS).NE.KF) GO TO 2
        ENDDO
        IF(KB.EQ.KB0) THEN
          IBODY=KB
          MAT=MATER(KB)
          RETURN
        ELSE IF(KDGHT(KB,NX).GT.1) THEN
          KB0=KB
          GO TO 100
        ELSE
          IBODY=KB
          MAT=MATER(KB)
          RETURN
        ENDIF
    2 CONTINUE
      IBODY=NBODY+1
      MAT=0
      RETURN
      END
C  **************************************************************
C                       SUBROUTINE STEP
C  **************************************************************
      SUBROUTINE STEP(DS,DSEF,NCROSS)
C
C     This subroutine handles the geometrical part of the track
C  simulation. The particle starts from the point (X,Y,Z) and
C  travels a length DS in the direction (U,V,W) within the
C  material where it moves. When the track leaves the initial
C  material, the particle is stopped just after entering the next
C  material body (void regions with MAT=0 are crossed automati-
C  cally). Furthermore, when the particle arrives from a void
C  region, it is stopped just after entering the first material
C  body.
C
C  Input values (common /TRACK/):
C     X, Y, Z ... coordinates of the initial point,
C     U, V, W ... direction cosines of the displacement,
C     IBODY ..... body where the initial point is located,
C     MAT ....... material in body IBODY.
C
C  Input argument:
C     DS ........ path length to travel.
C
C  Output arguments:
C     DSEF....... travelled path length before leaving the
C                 initial material or completing the jump (less
C                 than DS if the track crosses an interface),
C     NCROSS .... = 0 if the whole step is contained in the
C                   initial material,
C                 .gt.0 if the particle has crossed an interface,
C                   i.e. if it has entered a new material.
C
C  Output values (common /TRACK/):
C     X, Y, Z ... coordinates of the final position,
C     IBODY ..... body where the final point is located,
C     MAT ....... material in IBODY. The value MAT=0 indicates
C                 that the particle has escaped from the system.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      PARAMETER (NS=250,NB=125,NX=100)
      PARAMETER(NS2=2*NS,EPS=1.0D-8,ONE=1.0D0+EPS)
      DIMENSION S(NS2),KCHS(NS2)
      COMMON/QSURF/AXX(NS),AXY(NS),AXZ(NS),AYY(NS),AYZ(NS),
     1    AZZ(NS),AX(NS),AY(NS),AZ(NS),A0(NS),NSURF,KPLANE(NS)
      COMMON/QTREE/NBODY,MATER(NB),KMOTH(NB),KDGHT(NB,NX),
     1    KSURF(NB,NX),KFLAG(NB,NX),KSP(NS),KSLAST
C
      COMMON/TRACK/E,X,Y,Z,U,V,W,WGHT,KPAR,IBODY,MAT,ILB
C
      DO KS=1,NS
        KSP(KS)=0
      ENDDO
      DO KS=1,NS2
        S(KS)=0.0D0
        KCHS(KS)=0
      ENDDO
      DST=0.0D0
      DSEF=0.0D0
      NCROSS=0
      MAT0=MAT
      KC=0
      KMARK=0
      IF(MAT.EQ.0) THEN
        DSI=1.0D35
      ELSE
        DSI=DS
      ENDIF
      IF(IBODY.GT.NBODY) THEN
        KB1=NBODY
        GO TO 200
      ENDIF
C
C  ************  Surface crossings.
C
      KB0=IBODY
      SMIN=1.0D35
  100 CONTINUE
      NST=KSURF(KB0,NX)
      DO 1 KSS=1,NST
        KFL=KFLAG(KB0,KSS)
        KS=KSURF(KB0,KSS)
        IF(KPLANE(KS).EQ.0) THEN
          A=U*(AXX(KS)*U+AXY(KS)*V+AXZ(KS)*W)
     1     +V*(AYY(KS)*V+AYZ(KS)*W)+W*AZZ(KS)*W
          XXX=AXX(KS)*X+AXY(KS)*Y+AXZ(KS)*Z+AX(KS)
          YYY=AYY(KS)*Y+AYZ(KS)*Z+AY(KS)
          ZZZ=AZZ(KS)*Z+AZ(KS)
          B=U*(AXX(KS)*X+XXX)+V*(AXY(KS)*X+AYY(KS)*Y+YYY)
     1     +W*(AXZ(KS)*X+AYZ(KS)*Y+AZZ(KS)*Z+ZZZ)
          C=X*XXX+Y*YYY+Z*ZZZ+A0(KS)
        ELSE
          A=0.0D0
          B=U*AX(KS)+V*AY(KS)+W*AZ(KS)
          C=X*AX(KS)+Y*AY(KS)+Z*AZ(KS)+A0(KS)
        ENDIF
C
        IF(C.LT.0.0D0) THEN
          KSP(KS)=1
        ELSE
          KSP(KS)=2
        ENDIF
C  ****  Consistency test, and position correction if not passed.
        IF(KFL.LT.3.AND.KFL.NE.KSP(KS)) THEN
          CALL LOCATE
          IF(MAT.EQ.MAT0) GO TO 100
          DSEF=EPS
          NCROSS=1
          KSLAST=KS
          RETURN
        ENDIF
C
        IF(DABS(A).LT.1.0D-16) THEN
          IF(DABS(B).GT.1.0D-16*DABS(C)) THEN
            S1=-C/B
            IF(S1.GT.0.0D0) THEN
              IF(S1.LT.SMIN) THEN
                SMIN=S1
              ENDIF
              KC=KC+1
              S(KC)=S1
              KCHS(KC)=KS
            ENDIF
          ENDIF
        ELSE
          DISCR=B*B-4.0D0*A*C
          IF(DISCR.GT.1.0D-14) THEN
            D=DSQRT(DISCR)
            S1=(-B-D)/(A+A)
            IF(S1.GT.0.0D0) THEN
              IF(S1.LT.SMIN) THEN
                SMIN=S1
              ENDIF
              KC=KC+1
              S(KC)=S1
              KCHS(KC)=KS
            ENDIF
            S2=(-B+D)/(A+A)
            IF(S2.GT.0.0D0) THEN
              IF(S2.LT.SMIN) THEN
                SMIN=S2
              ENDIF
              KC=KC+1
              S(KC)=S2
              KCHS(KC)=KS
            ENDIF
          ENDIF
        ENDIF
    1 CONTINUE
C
C  ****  The particle remains in the same body.
C
      IF(MAT.NE.0.AND.DSI.LT.SMIN) THEN
        NCROSS=0
        DSEF=DSI
        X=X+DSEF*U
        Y=Y+DSEF*V
        Z=Z+DSEF*W
        RETURN
      ENDIF
C
C  ************  Surface crossings (sister bodies and modules).
C
      KB1=KMOTH(KB0)
      IF(KB1.EQ.0) THEN
        KB1=KB0
        GO TO 3
      ENDIF
  200 NST=KSURF(KB1,NX)
      DO 2 KSS=1,NST
        KS=KSURF(KB1,KSS)
        IF(KSP(KS).NE.0) GO TO 2
        IF(KPLANE(KS).EQ.0) THEN
          A=U*(AXX(KS)*U+AXY(KS)*V+AXZ(KS)*W)
     1     +V*(AYY(KS)*V+AYZ(KS)*W)+W*AZZ(KS)*W
          XXX=AXX(KS)*X+AXY(KS)*Y+AXZ(KS)*Z+AX(KS)
          YYY=AYY(KS)*Y+AYZ(KS)*Z+AY(KS)
          ZZZ=AZZ(KS)*Z+AZ(KS)
          B=U*(AXX(KS)*X+XXX)+V*(AXY(KS)*X+AYY(KS)*Y+YYY)
     1     +W*(AXZ(KS)*X+AYZ(KS)*Y+AZZ(KS)*Z+ZZZ)
          C=X*XXX+Y*YYY+Z*ZZZ+A0(KS)
        ELSE
          A=0.0D0
          B=U*AX(KS)+V*AY(KS)+W*AZ(KS)
          C=X*AX(KS)+Y*AY(KS)+Z*AZ(KS)+A0(KS)
        ENDIF
C
        IF(C.LT.0.0D0) THEN
          KSP(KS)=1
        ELSE
          KSP(KS)=2
        ENDIF
C
        IF(DABS(A).LT.1.0D-16) THEN
          IF(DABS(B).GT.1.0D-16*DABS(C)) THEN
            S1=-C/B
            IF(S1.GT.0.0D0) THEN
              KC=KC+1
              S(KC)=S1
              KCHS(KC)=KS
            ENDIF
          ENDIF
        ELSE
          DISCR=B*B-4.0D0*A*C
          IF(DISCR.GT.1.0D-14) THEN
            D=DSQRT(DISCR)
            S1=(-B-D)/(A+A)
            IF(S1.GT.0.0D0) THEN
              KC=KC+1
              S(KC)=S1
              KCHS(KC)=KS
            ENDIF
            S2=(-B+D)/(A+A)
            IF(S2.GT.0.0D0) THEN
              KC=KC+1
              S(KC)=S2
              KCHS(KC)=KS
            ENDIF
          ENDIF
        ENDIF
    2 CONTINUE
    3 CONTINUE
      IF(KC.EQ.0) GO TO 10
C
C  ****  Sort surface distances in decreasing order.
C
      DO 5 KI=1,KC-1
      SMAX=S(KI)
      KMAX=KI
      DO 4 KJ=KI+1,KC
      IF(S(KJ).GT.SMAX) THEN
        SMAX=S(KJ)
        KMAX=KJ
      ENDIF
    4 CONTINUE
      IF(KMAX.NE.KI) THEN
        SMAX=S(KI)
        S(KI)=S(KMAX)
        S(KMAX)=SMAX
        KKMAX=KCHS(KI)
        KCHS(KI)=KCHS(KMAX)
        KCHS(KMAX)=KKMAX
        IF(KMARK.EQ.KI) THEN
          KMARK=KMAX
        ELSE IF(KMARK.EQ.KMAX) THEN
          KMARK=KI
        ENDIF
      ENDIF
    5 CONTINUE
C
C  ************  New position.
C
      KCN=KC
      KMARK0=KMARK
      DO 9 KI=KCN,1,-1
      IF(MAT.EQ.MAT0.AND.S(KI).GT.DSI) THEN
        DSEF=DSI
        DST=DSI
        GO TO 11
      ENDIF
      IF(KI.NE.KMARK0) THEN
        KS=KCHS(KI)
        IF(KSP(KS).EQ.1) THEN
          KSP(KS)=2
        ELSE
          KSP(KS)=1
        ENDIF
C  ***   When the particle crosses an interface, it is shifted
C        an additional small length to avoid being trapped.
        DST1=ONE*S(KI)+EPS
        IF(DST1.LT.DST) GO TO 8
        IF(KI.GT.1) THEN
          IF(DST1.GT.S(KI-1)) GO TO 8
        ENDIF
        DST=DST1
      ENDIF
C  ****  KMARK indicates the crossing that is active.
      KMARK=KI
C
C  ****  Locate the new position.
C
      NDT=KDGHT(KB1,NX)
      DO 6 KBB=1,NDT
        KB=KDGHT(KB1,KBB)
        NSV=KSURF(KB,NX)
        DO KSS=1,NSV
          KS=KSURF(KB,KSS)
          KF=KFLAG(KB,KSS)
          IF(KF.LT.3.AND.KSP(KS).NE.KF) GO TO 6
        ENDDO
        IF(KB.EQ.KB1) THEN
          IBODY=KB
          MAT=MATER(KB)
          GO TO 7
        ELSE IF(KDGHT(KB,NX).GT.1) THEN
          KB1=KB
          GO TO 200
        ELSE
          IBODY=KB
          MAT=MATER(KB)
          GO TO 7
        ENDIF
    6 CONTINUE
      KBT=KMOTH(KB1)
      IF(KBT.GT.0) THEN
        KB1=KBT
        GO TO 200
      ENDIF
      IBODY=NBODY+1
      MAT=0
C
    7 CONTINUE
C  ****  The particle continues flying when it enters a void
C        region...
      IF(MAT.EQ.0) THEN
        IF(MAT0.NE.0) THEN
          NCROSS=NCROSS+1
          IF(NCROSS.EQ.1) DSEF=DST
        ENDIF
        MAT0=0
        DSI=1.0D30
        GO TO 8
      ENDIF
C  ****  ... and stops when it penetrates a new material body.
      IF(MAT.NE.MAT0) THEN
        NCROSS=NCROSS+1
        IF(NCROSS.EQ.1) DSEF=DST
        IF(MAT.NE.0) GO TO 11
      ENDIF
C  ****  Crossings that have been passed are discarded.
    8 KC=KC-1
    9 CONTINUE
C
   10 CONTINUE
      IF(MAT.EQ.0) THEN
        DST=1.0D30
        IBODY=NBODY+1
      ELSE
        DSEF=DSI
        DST=DSI
      ENDIF
C
   11 X=X+DST*U
      Y=Y+DST*V
      Z=Z+DST*W
      KSLAST=KCHS(KMARK)
      RETURN
      END
